Subroutine driver_michelsen(rhox, pbb, ybb, fvalue_uminf, iparam2, rparam2, fugacity_z)


!	Driver that invokes Michelsen's Method.  Note that Michelsen's routine
!	here is set up only to check stability of a one phase system.  
!	This should suffice because if a 3 phase system is stable, each one of
!	the 3 phases should be stable as well.	As per Michelsen, we will use 
!	several different initial guesses to start his algorithm.

!	YOU SHOULD MODIFY THE INITIAL GUESS INTO MICHELSEN'S ALGORITHM IN THIS
!	SUBROUTINE  

!	The number of initial estimates should correspond to the number of components in system.
!	The initial estimates for 2 phases stability are:	
!								(1) mole_num(i) = exp(h(i))
!								(2) mole_num(i) = 1/2(y(i) + x(i))
!								(3) mole_num(i) = (1-0.99999)/(ncp-1) for i .ne. s where s is
!										the x(i) of heaviest and then most volatile component
!	Initial estimates for 1 phase system will be calculated using a grid system.
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			4/02/00		P.D. Ting				inception
!

use essentials
Implicit None

!	Declaring variables that are passed in/out of this subroutine
real (KIND=double), intent(in) :: rhox			! number density of molecules (per ang^3)
real (KIND=double), intent(out) :: fvalue_uminf		! michelsen difference
real (KIND=double), intent(out), dimension(ncp) :: pbb
real (KIND=double), intent(out) :: ybb
integer, dimension(7), intent(in) :: iparam2
real (KIND=double), dimension(7), intent(in) :: rparam2

!	Declaring variables used in this subroutine by Thermoprops
real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp) :: x, xd,xq,mu,q
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
integer, dimension(ncp) :: N_sites
real (KIND=double) :: Prsm						! total residual P
real (KIND=double) :: Pcalc						! calculated Pressure in bars
real (KIND=double) :: Ursm						! total residual U
real (KIND=double) :: Arsm						! total residual A
real (KIND=double) :: Grsm						! total residual G
real (KIND=double) :: Srsm						! total residual S
real (KIND=double) :: Hrsm						! total residual H
real (KIND=double) :: Gtotal					! Total Gibbs free energy of mixture
real (KIND=double), dimension(ncp) :: chpt		! total chemical potential
real (KIND=double), dimension(ncp) :: chpt_res	! residual total chemical potential
real (KIND=double), dimension(ncp) :: fugacity	! natural log of fugacity coefficient

!	Declaring variables used by this subroutine
real (KIND=double), intent(out), dimension(ncp) :: fugacity_z	! feed fugacity
real (KIND=double), dimension(ncp) :: mole_frac		! system mole frac.
real (KIND=double), dimension(ncp) :: mole_num_guess ! system mole frac.
real (KIND=double), dimension(ncp) :: mole_num
real (KIND=double) :: sum
integer :: i,j

!	Declared variables used by IMSL conditioning statements
real (KIND=double) :: fscale2
real (KIND=double), dimension(ncp) :: mole_num_scale

external michelsen_mini, dimension_convert, PTthermo_props, duminf


!	Call thermoprops to obtain the fugacities needed to set up the inital
!	Michelsen input parameters


call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&fb, N_sites,q,mu,xq,xd)

 forall (i=1:ncp)
  x(i) = z(i)
end forall

write(6,*) 'before calling PTT for the first time'
write(6,*) 'rhox is',rhox
write(6,*) 'ncp is',ncp
write(6,*) 'max is', max
write(6,*) 'status11 is', status11
write(6,*) 'm(1) is',m(1)
write(6,*) 'm(2) is',m(2)
write(6,*) 'x(1) is',x(1)
write(6,*) 'x(2) is',x(2)
write(6,*) 'u0(1,1) is',u0(1,1)
write(6,*) 'u0(1,2) is',u0(1,2)
write(6,*) 'pc dispersion is',PC_dispersion

! pause
call PTthermo_props(ncp, max, status11, TK, rhox, m, x, u0, e_ii, &
		&	kij, N_sites, fa, fb, kappa, sig, D, v00, Prsm, &
		&	Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, Pcalc, fugacity, chpt_res, Gtotal,PC_dispersion,&
		&   analytic_flag, polar_flag, Q, mu, xd, xq)


!	Setting up IMSL required parameters
fscale2 = 1.0
forall (i=1:ncp)
  mole_num_scale(i) = 1.0
end forall


!	Assign the above to the input thermo properties of the flash.  
forall (i=1:ncp)
	z_chpt(i) = chpt(i)
	fugacity_z(i) = fugacity(i)
end forall

ybb = 10

forall (i=1:ncp)
	h(i) = dlog( z(i) ) + fugacity_z(i) 
end forall


forall (i=1:ncp)
	mole_frac(i) = 100.0
end forall


!	Conditioning the 1st set of initial guess into Michelsen's routine

forall (i=1:ncp)
  mole_num_guess(i) = dexp(h(i))
end forall

sum = 0.0d0

call duminf(michelsen_mini, ncp, mole_num_guess, mole_num_scale, fscale2, & 
	&	iparam2, rparam2, mole_num, fvalue_uminf) 


!	If fvalue_uminf is less than 0, then the system is unstable and we

if (fvalue_uminf .lt. -1.0d-17) then
  write(6,*) 'Feed system is unstable'
		
  sum = 0.0d0
  do i = 1, ncp
	sum = sum + mole_num(i)
  end do

  forall (i = 1:ncp)
	mole_frac(i) = mole_num(i) / sum
  end forall

  forall (i=1:ncp)	
  	pbb(i) = mole_frac(i)
  end forall
  ybb = fvalue_uminf

end if



!	Conditioning the 2nd set of initial guess

do j = 1, ncp
  if (fvalue_uminf .gt. -1.0d-17) then

    forall (i=1:ncp, i .ne. j)
	  mole_num_guess(i) = (1-0.9999)/(ncp-1) 
    end forall

    mole_num_guess(j) = 0.9999

    sum = 0.0d0

    call duminf(michelsen_mini, ncp, mole_num_guess, mole_num_scale, 1, iparam2, rparam2, &
	  & mole_num, fvalue_uminf) 

!    write(6,*) 'minimum of g_star (negative = unstable)', fvalue_uminf

    if (fvalue_uminf .lt. -1.0d-17) then

  	  sum = 0.0d0
	    do i = 1, ncp
		  sum = sum + mole_num(i)
	    end do
	
	    do i = 1,ncp
		  mole_frac(i) = mole_num(i) / sum
	    end do

	    forall (i=1:ncp)
		  pbb(i) = mole_frac(i)
	    end forall
	    ybb = fvalue_uminf

    end if

  end if
end do

!	Conditioning the 3rd set of initial guesses

do j = 1, ncp
  if (fvalue_uminf .gt. -1.0d-17) then

    forall (i=1:ncp, i .ne. j)
	  mole_num_guess(i) = (1-0.9999999)/(ncp-1) 
	end forall

	mole_num_guess(j) = 0.9999999

  	sum = 0.0d0

	call duminf(michelsen_mini, ncp, mole_num_guess, mole_num_scale, 1, iparam2, rparam2, &
				& mole_num, fvalue_uminf) 

	if (fvalue_uminf .lt. -1.0d-17) then

	  sum = 0.0d0
	  do i = 1, ncp
		sum = sum + mole_num(i)
	  end do
	
	  do i = 1,ncp
		mole_frac(i) = mole_num(i) / sum
	  end do

	  forall (i=1:ncp)
		pbb(i) = mole_frac(i)
	  end forall
	  ybb = fvalue_uminf

	end if

  end if
end do

!	Conditioning the 4th set of initial guesses
do j = 1, ncp
 
  if (fvalue_uminf .gt. -1.0d-17) then

	forall (i=1:ncp, i .ne. j)
	  mole_num_guess(i) = (1-0.333)/(ncp-1) 
	end forall

	mole_num_guess(j) = 0.333

	call duminf(michelsen_mini, ncp, mole_num_guess, mole_num_scale, 1, iparam2, rparam2, &
			& mole_num, fvalue_uminf) 

	if (fvalue_uminf .lt. -1.0d-17) then

	  sum = 0.0d0
	  do i = 1, ncp
 		sum = sum + mole_num(i)
	  end do

	  do i = 1,ncp
		mole_frac(i) = mole_num(i) / sum
	  end do
	  forall (i=1:ncp)
		pbb(i) = mole_frac(i)
	  end forall
	  ybb = fvalue_uminf

	end if


  end if
end do
	
!	Conditioning the 5th set of initial guesses
do j = 1, ncp
  if (fvalue_uminf .gt. -1.0d-17) then
	forall (i=1:ncp, i .ne. j)
	  mole_num_guess(i) = (1.0-0.85)/(ncp-1) 
	end forall

	mole_num_guess(j) = 0.85


	call duminf(michelsen_mini, ncp, mole_num_guess, mole_num_scale, 1, iparam2, rparam2, &
			& mole_num, fvalue_uminf) 

	if (fvalue_uminf .lt. -1.0d-17) then

	  sum = 0.0d0
	  do i = 1, ncp
		sum = sum + mole_num(i)
 	  end do

	  do i = 1,ncp
		mole_frac(i) = mole_num(i) / sum
	  end do
	  forall (i=1:ncp)
		pbb(i) = mole_frac(i)
	  end forall
	  ybb = fvalue_uminf

	end if


  end if
end do


END SUBROUTINE