subroutine michelsen_mini(N2, input, output)		

!	The argument list is conditioned for DUMINF

!	This subroutine implements Michelsen's stability routine given in:
!
!		Michael Michelsen.  'The isothermal flash problem, Part I:
!		Stability.' Fluid Phase Equilibria. 9 (1982) p. 1-19
!
!	The subroutine takes h, the feed (fugacity/pressure).
!	Input is the estimate of Y(i)s (molnum(i)s) 
!	Michelsen_mini differs from Michelsen_driver in that Michelsen_mini
!	is reformulated as a minimization routine of the function:
!
!		g_star(Y) = 1 + sum(Y(i) *(lnY(i) + fugacity(i) - h(i) - 1) 
!
!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			4/02/00		P.D. Ting				inception
!


use essentials
implicit none

!	Declaring passing variables
real (KIND=double), dimension(ncp), intent(in) :: input		! Initial guess of Y(i)s (mole numbers)
real (KIND=double), intent(out) :: output	! Solution mole numbers
integer, intent(in) :: N2									! Number of equations to be solved 

!	Declaring local variables
real (KIND=double), dimension(ncp) :: mole_num				! mole numbers 
integer :: i, flag, flag1
real (KIND=double), dimension(ncp) :: y						! test real mole fraction obtained from 
															!	mole numbers
real (KIND=double) :: rhox									! number density of molecules
real (KIND=double) :: sum

!	Declaring thermoprops variables
!	Declaring variables used in this subroutine by Thermoprops
real (KIND=double), dimension(ncp) :: m
real (KIND=double), dimension(ncp,ncp) :: u0
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(ncp,ncp,max,max) :: kappa
real (KIND=double), dimension(ncp,ncp) :: sig
real (KIND=double), dimension(ncp) :: v00
real (KIND=double), dimension(ncp,ncp) :: kij
real (KIND=double), dimension(ncp) :: x, xd, xq, Q, mu
real (KIND=double), dimension(ncp,ncp,max,max,2) :: fa
real (KIND=double), dimension(ncp,ncp,max,max,1) :: fb
integer, dimension(ncp) :: N_sites
real (KIND=double) :: Prsm						! total residual P
real (KIND=double) :: Pcalc						! calculated Pressure in bars
real (KIND=double) :: Ursm						! total residual U
real (KIND=double) :: Arsm						! total residual A
real (KIND=double) :: Grsm						! total residual G
real (KIND=double) :: Srsm						! total residual S
real (KIND=double) :: Hrsm						! total residual H
real (KIND=double) :: Gtotal					! Total Gibbs free energy of mixture
real (KIND=double), dimension(ncp) :: chpt		! total chemical potential
real (KIND=double), dimension(ncp) :: chpt_res	! residual total chemical potential
real (KIND=double), dimension(ncp) :: fugacity	! natural log of fugacity coefficient


!	Here's the meat

forall (i=1:ncp)
	mole_num(i) = input(i)
end forall
flag = 0


!	Making sure we don't get negative mole numbers
do i = 1, ncp
  if (mole_num(i) .lt. 0.0d0) then
	flag  = 1
  end if
end do

!	Converint mole numbers to mole fractions
if (flag .ne. 1) then

  sum = 0.0d0
  do i = 1, ncp
	sum = sum + mole_num(i)
  end do

  forall (i=1:ncp)
	y(i) = mole_num(i) / sum
  end forall

!	Calculating the fugacity coefficients corresponding to the mole fraction and mole number

  call liquid_or_vapor(rhox, y)
  call dimension_convert(m, u0, e_ii, kappa, sig, v00, kij, x, fa, &
		&fb, N_sites,q,mu,xq,xd)

  call PTthermo_props(ncp, max, status11, TK, rhox, m, x, u0, e_ii, kij, & 
					&	N_sites, fa, fb, kappa, sig, D, v00, Prsm, Ursm, & 
					&	Arsm, Grsm, Srsm, Hrsm, chpt, Pcalc, fugacity, &
					&	chpt_res, Gtotal, PC_dispersion, analytic_flag,&
						& polar_flag, Q, mu, xd, xq ) 

!	Calculating the output. 
  output = 0.0d0
  do i = 1, ncp	
	output = output + mole_num(i)*(dlog(mole_num(i)) + fugacity(i) - h(i) - 1)
  end do
  output = output + 1 

end if


if (flag .eq. 1) then	
  output = 10000.0
end if

end subroutine