subroutine test_PCdispersion(ncp, sig, u0, TK, kij, rhox, zeta3, Amf, Pmf, Gmf, &
				& Hmf, Smf, Umf, chptlmf, x, m, dhs)
!	Subroutine dispersion
!
!	Program written by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Calculate the dispersion (mean field) contribution of the hard sphere mixtures.
!		This is designated as part of the segment contribution terms (dispersion + HS)
!		We will use vdW-One conformal fluid to deal with mixture effects.  The dispersion 
!		will be calculated using fitted parameters by Gross and Sadowski.  
!
!		This code calculates:
!				residual mean field pressure [p/rho kT]
!				residual mean field internal energy [U/NkT]
!				residual mean field Helmholtz free energy [A/NkT]
!				residual mean field Gibbs free energy [G/NkT]
!				residual mean field entropy [S/Nk]
!				residual mean field enthalpy [H/NkT]
!				residual chemical mean field potential of components [chptl/kT]
!			
!			where N is the number of CHAINS
!
!INPUTS
!		REAL
!			ncp							!number of components
!			TK							! Temperature in Kelvin
!			rhox						! # molecules per volume
!			zeta3						! zeta3
!	ARRAYS and VECTORS	
!			dimension(ncp,ncp) :: sig	!temp. ind. sigma
!			dimension(ncp, ncp) :: kij	! k(i,j), fitting parameter 
!			dimension(ncp) :: x			! mole fraction of component i
!			dimension(ncp) :: m			! chain length of molecule i
!			dimension(ncp,ncp) :: u0 	! (u0_i,i)/k in K 
!			dimension(ncp) :: dhs		! temp. dependent. hs diameter
!OUTPUTS
!	REAL
!		Amf						! mean field Helmholtz free energy
!		Pmf						! mean field pressure
!		Gmf						! mean field Gibbs free energy
!		Hmf						! mean field Enthalpy
!		Smf						! mean field entropy
!		Umf						! mean field internal energy
!	ARRAYS
!		dimension(ncp) :: chptlmf	! mean field res. chemical potential
!	
! Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			6/09/00		P.D. Ting				inception 
!use essentials 
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp									! number of components
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig	! temp. ind. sigma
real (KIND=double), intent(in), dimension(ncp) :: dhs		! temp. dependent. hs diameter
real (KIND=double), intent(in), dimension(ncp, ncp) :: u0	! (u0_i,i)/k in K 
real (KIND=double), intent(in), dimension(ncp, ncp) :: kij	! k(i,j)
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in) :: TK						! Temperature in Kelvin
real (KIND=double), intent(in) :: rhox						! # molecules per volume
real (KIND=double), intent(in) :: zeta3						! zeta3
real (KIND=double), intent(out) :: Amf						! mean field Helmholtz free energy
real (KIND=double), intent(out) :: Pmf						! mean field pressure
real (KIND=double), intent(out) :: Gmf						! mean field Gibbs free energy
real (KIND=double), intent(out) :: Hmf						! mean field Helmholtz free energy
real (KIND=double), intent(out) :: Smf						! mean field entropy
real (KIND=double), intent(out) :: Umf						! mean field internal energy
real (KIND=double), intent(out), dimension(ncp) :: chptlmf	! mean field res. chemical potential


!	Declaring local parameters
real (KIND=double), dimension(7,3) :: ap, bp				! PCSaft dispersion parameters
real (KIND=double) , dimension(ncp, ncp) :: u				! temperature dependent u
real (KIND=double) :: rho									! density in molecules/ang^3
real (KIND=double) :: m_sum
real (KIND=double), dimension(ncp) :: e_ii
real (KIND=double), dimension(7) :: apar, bpar				! dispersion parameters ai(m) and bi(m)
real (KIND=double), dimension(ncp, 7) :: dap_dx, dbp_dx		! derivatives of apar and bpar
real (KIND=double) :: Z00, z00_dz							! contributions to Amf calculations
real (KIND=double) :: drdpkt								! C1 in Joachim's code
real (KIND=double) :: drdpdz								! C2 in Joachim's code
real (KIND=double) :: edI1dz, edI2dz						! d (zeta3*I1)/d(zeta3)
real (KIND=double) :: zms									! 1 - zeta3
real (KIND=double) :: z00_d2, ddendv, fdspdz													
real (KIND=double) :: order1, order2						! vdw mixing rule parameters
real (KIND=double), dimension(ncp) :: ddendx
real (KIND=double) :: D, I1, I2
real (KIND=double), dimension(ncp) :: I1_dx, I2_dx, ord1dx, ord2dx
real (KIND=double), dimension(ncp) :: dfdx
real (KIND=double) :: myres, ddrdx, dte3dx, dte2dx, dte1dx, te1, te2, te3
real (KIND=double) :: dzeta3dT, dI1dT, dI2dT, dC1dT
real (KIND=double) :: dAmfdt

real (KIND=double) :: z3,eta,a_disp,z_disp, C1, C2

! variables in infinite dilution chemical potential computation

real (KIND=double), dimension(ncp) :: d_adisp_dx, mu_res, C1_x, eta_x
real (KIND=double), dimension(7,ncp) :: ap_x,bp_x
integer :: i,j, k 


!	Declaring local parameters
real (KIND=double), parameter :: tau = 0.7404804897			! pi/6*sqrt(2)
real (KIND=double), parameter :: pi = 3.141592654d0

!	PC SAFT dispersion parameters
ap(1,1)=  0.91056314451539d0
ap(1,2)= -0.30840169182720d0
ap(1,3)= -0.09061483509767d0
ap(2,1)=  0.63612814494991d0
ap(2,2)=  0.18605311591713d0
ap(2,3)=  0.45278428063920d0
ap(3,1)=  2.68613478913903d0
ap(3,2)= -2.50300472586548d0
ap(3,3)=  0.59627007280101d0
ap(4,1)= -26.5473624914884d0
ap(4,2)=  21.4197936296668d0
ap(4,3)= -1.72418291311787d0
ap(5,1)=  97.7592087835073d0
ap(5,2)= -65.2558853303492d0
ap(5,3)= -4.13021125311661d0
ap(6,1)= -159.591540865600d0
ap(6,2)=  83.3186804808856d0
ap(6,3)=  13.7766318697211d0
ap(7,1)=  91.2977740839123d0
ap(7,2)= -33.7469229297323d0
ap(7,3)= -8.67284703679646d0

bp(1,1)=  0.72409469413165d0
bp(1,2)= -0.57554980753450d0
bp(1,3)=  0.09768831158356d0
bp(2,1)=  1.11913959304690d0  *2.d0
bp(2,2)=  0.34975477607218d0  *2.d0
bp(2,3)= -0.12787874908050d0  *2.d0
bp(3,1)= -1.33419498282114d0  *3.d0
bp(3,2)=  1.29752244631769d0  *3.d0
bp(3,3)= -3.05195205099107d0  *3.d0
bp(4,1)= -5.25089420371162d0  *4.d0
bp(4,2)= -4.30386791194303d0  *4.d0
bp(4,3)=  5.16051899359931d0  *4.d0
bp(5,1)=  5.37112827253230d0  *5.d0
bp(5,2)=  38.5344528930499d0  *5.d0
bp(5,3)= -7.76088601041257d0  *5.d0
bp(6,1)=  34.4252230677698d0  *6.d0
bp(6,2)= -26.9710769414608d0  *6.d0
bp(6,3)=  15.6044623461691d0  *6.d0
bp(7,1)= -50.8003365888685d0  *7.d0
bp(7,2)= -23.6010990650801d0  *7.d0
bp(7,3)= -4.23812936930675d0  *7.d0

!	Converting density from dimensionless to molecules/ang^3
rho = rhox/(sig(1,1)**3)

!	Note that e_ii is set to 0 for PC SAFT
m_sum = 0.0D0


do i = 1, ncp
  e_ii(i)=0.d0
  m_sum = m_sum + x(i) * m(i)
  u(i,i) = u0(i,i)
end do

do i = 1, ncp
  do j = 1, ncp
	  u(i,j) = (1.D0 - kij(i,j)) * dsqrt( u(i,i) * u(j,j) )
  end do
end do


!	Dispersion coefficients calculation

do j=1,7
  apar(j) = ap(j,1) + (1.d0-1.d0/m_sum)*ap(j,2) + (1.d0-1.d0/m_sum)* &
            &  (1.d0-2.d0/m_sum)*ap(j,3)

  bpar(j) = bp(j,1) + (1.d0-1.d0/m_sum)*bp(j,2) + (1.d0-1.d0/m_sum)* &
			&  (1.d0-2.d0/m_sum)*bp(j,3)
end do


!	Derivatives of dispersion coefficients wrt mole fraction x(i)

do k=1,ncp
  do j=1,7
    dap_dx(k,j) = m(k)/m_sum**2.d0*ap(j,2) &
     &      +(3.d0*m(k)/m_sum**2.d0- 4.d0*m(k)/m_sum**3.d0)*ap(j,3)

    dbp_dx(k,j) = m(k)/m_sum**2.d0*bp(j,2) & 
     &      +(3.d0*m(k)/m_sum**2.d0- 4.d0*m(k)/m_sum**3.d0)*bp(j,3)
  end do
end do

!	Determining the vdw mixing rule avg(m^2*u0*sig^3) and avg(m^2*u0^2*sig^3)

order1 = 0.d0
order2 = 0.d0
do i = 1,ncp
  do j = 1,ncp
    order1 = order1 + x(i) * x(j) * m(i) * m(j) * sig(i,j)**3.d0 * u(i,j)/TK
    order2 = order2 + x(i) * x(j) * m(i) * m(j) * sig(i,j)**3.d0 * (u(i,j)/TK)**2.d0
  end do
end do


I1    = 0.d0
I2    = 0.d0
edI1dz = 0.d0
edI2dz = 0.d0

!	Calculating I1 I2 and d(zeta3*I1)/dzeta3, d(zeta3*I2)/dzeta3

do j = 0,6
  I1  = I1 + apar(j+1) * zeta3**DBLE(j)
  I2  = I2 + bpar(j+1) * zeta3**DBLE(j)
  edI1dz = edI1dz + apar(j+1)*DBLE(j+1)*zeta3**DBLE(j)
  edI2dz = edI2dz + bpar(j+1)*DBLE(j+1)*zeta3**DBLE(j)
end do


D=zeta3/((pi/6.d0)*rho)

do i=1,ncp
  ddendx(i) = zeta3/D*m(i)*dhs(i)**3.d0
  I1_dx(i) = 0.d0
  I2_dx(i) = 0.d0

  do j = 0,6
    I1_dx(i)= I1_dx(i) + apar(j+1)*DBLE(j)*zeta3**DBLE(j-1)*ddendx(i) &
     &             + dap_dx(i,j+1)*zeta3**DBLE(j)

    I2_dx(i)= I2_dx(i) + bpar(j+1)*DBLE(j)*zeta3**DBLE(j-1)*ddendx(i) &
     &             + dbp_dx(i,j+1)*zeta3**DBLE(j)
  end do

  ord1dx(i)  = 0.d0
  ord2dx(i)  = 0.d0
  do j = 1,ncp
          ord1dx(i)= ord1dx(i) + 2.d0 * m(i) * x(j) * m(j) * sig(j,i)**3.d0 * u(j,i)/TK
          ord2dx(i)= ord2dx(i) + 2.d0 * m(i) * x(j) * m(j) * sig(j,i)**3.d0  &
		  &			*(u(j,i)/TK)**2.d0
  end do
end do


!	Calculating the Helmholtz free energy contribution
!	Note that C1 = drdpkt

zms = 1.d0-zeta3

Z00    = 1.d0 + m_sum*(4.d0*zeta3-2.d0*zeta3**2.d0)/(zms**3.d0)  &
     &         + (1.d0 - m_sum)*(5.d0*zeta3-2.d0*zeta3**2.d0)    &
     &                               /(zms*(2.d0-zeta3))

z00_dz = m_sum*(-2.d0*zeta3**2.d0+4.d0*zeta3+4.d0)/zms**4.d0     &
     &         + (1.d0 - m_sum)*(zeta3**2.d0-8.d0*zeta3+10.d0)   &
     &                                /(zms*(2.d0-zeta3))**2.d0
drdpkt = 1.d0/(Z00 + zeta3*z00_dz)

Amf  = -2.d0 * pi * rho * I1 * order1 - pi * rho * drdpkt * m_sum * I2 * order2



!	Calculating the compressibility contribution
!	Calculating C2=dC1/d zeta3 (drdpdz)
z00_d2 = m_sum*(-4.d0*zeta3**2.d0+8.d0*zeta3+20.d0)/ZMS**5.d0      &
     &        + (1.d0 - m_sum)                                     &
     &        *(-2.d0*zeta3**3.d0+24.d0*zeta3**2.d0-60.d0*zeta3+44.d0)   &
     &                                /(ZMS*(2.d0-zeta3))**3.d0 

drdpdz = (-2.d0*z00_dz -z00_d2*zeta3)*drdpkt**2.d0


		! be aware: rho=6.d0*zeta*3zeta3/(PI*D)
      fdspdz = -2.d0*PI  *6.d0/(PI*D) * edI1dz * order1                &
     &         -( PI *6.d0*zeta3/(PI*D) *drdpdz* m_sum *I2 *order2     &
     &           +PI *6.d0/(PI*D)   *drdpkt* m_sum *edI2dz *order2 )
      ddendv = zeta3**2.d0  *6.d0/ PI /D
Pmf  = ddendv* fdspdz / rho


!	Calculating the fugacity coefficient

do i = 1, ncp
  te1 = zms**4.d0 *(2.d0-zeta3)**2.d0
  te2 = zms**2.d0 * (  zms**2.d0*(2.d0-zeta3)**2.d0 +zeta3*(-2.d0*zeta3**3.d0+  &
		&		12.d0 * zeta3**2.d0 - 27.d0 * zeta3 + 20.d0)  )

  te3 = zeta3* (  (2.d0-zeta3)**2.d0*(8.d0-2.d0*zeta3)  &
     &         -zms**2.d0*(-2.d0*zeta3**3.d0  &
     &         +12.d0*zeta3**2.d0-27.d0*zeta3+20.d0)  )


  drdpkt = te1 / (te2 + m_sum*te3)

  dte1dx = ( - 4.d0*zms**3.d0 *(2.d0-zeta3)**2.d0     &
     &             - 2.d0*zms**4.d0 *(2.d0-zeta3) )*ddendx(i)

  dte2dx = dte1dx+ ( (3.d0*zeta3**2.d0 - 2.d0*2.d0*zeta3 + 1.d0)  &
     &             *(-2.d0*zeta3**3.d0+12.d0*zeta3**2.d0 -27.d0*zeta3+20.d0) &
     &             + zeta3*zms**2.d0*(-6.d0*zeta3**2.d0+24.d0*zeta3 -27.d0) )*ddendx(i)

  dte3dx =  ddendx(i)* ( te3/zeta3 + zeta3*( -2.d0*(2.d0-zeta3)*(8.d0-2.d0*zeta3)  &
     &             +(2.d0-zeta3)**2.d0*(-2.d0) + 2.d0*zms*(-2.d0*zeta3**3.d0    &
     &             +12.d0*zeta3**2.d0-27.d0*zeta3+20.d0) -zms**2.d0*(-6.d0*zeta3**2.d0  &
     &             +24.d0*zeta3 -27.d0) ) )


  ddrdx  = ( dte1dx*(te2+m_sum*te3) -te1*(dte2dx +(m_sum*dte3dx+m(i)*te3)) )  &
     &           / (te2 + m_sum*te3)**2.d0

  dfdx(i)= -2.d0*PI*rho*(order1*I1_dx(i)+ord1dx(i)*I1)     &
     &            -  PI*rho*drdpkt*m_sum*(order2*I2_dx(i)+ord2dx(i)*I2)    &
     &            -  PI*rho*(drdpkt*m(i)+ddrdx*m_sum)*order2*I2   

end do

myres=0.d0


do i=1, ncp
  myres=myres-x(i)* dfdx(i)
enddo

!	Calculating the chemical potential
do i=1,ncp
  chptlmf(i)=myres+Amf+Pmf  +dfdx(i)
enddo

!	Calculating the Gibbs free energy contribution

Gmf=0.0d0
do i=1,ncp
 Gmf = Gmf + x(i) * chptlmf(i)
end do


!	Calculate the entropy contribution from dAmfdT(v,x)
dzeta3dT = 0.0d0
dI1dT = 0.0d0
dI2dT = 0.0d0

do i = 1, ncp
  dzeta3dT = dzeta3dT + m(i) * x(i) * 3.0d0 * dhs(i)**2 * (-0.12) * sig(i,i) * &
		&		exp(-3*u0(i,i)/TK) * (3.0d0*u0(i,i)/TK/TK)
end do

dzeta3dT = pi/6.0d0*rho* dzeta3dT



do i = 0,6
  dI1dT = dI1dT + apar(i+1) * DBLE(i) * zeta3**(i-1)*dzeta3dT   

  dI2dT = dI2dT + bpar(i+1) * DBLE(i) * zeta3**(i-1)*dzeta3dT
end do

dC1dT = -drdpkt**2 * dzeta3dT * ( m_sum*(8.0 + 20.0*zeta3 - 4.0*zeta3**2) / ((1.0-zeta3)**5) &
	& + (1.0-m_sum) * (40.0-48.0*zeta3+12.0*zeta3**2 + 2.0*zeta3**3) / (((1.0-zeta3)*(2.0-zeta3))**3))
  

dAmfdt = -2 * pi * rho * order1 * (dI1dT - I1 / TK) - pi * rho * m_sum * order2 * &
			&  (dC1dT*I2 + drdpkt*dI2dT - 2*drdpkt*I2/Tk)





Smf = -TK*dAmfdT - Amf
Umf = Amf + Smf
Hmf = Umf + Pmf
!Gmf = Amf + Pmf
!write (6,*) 'exiting test PC dispersion exiting test PC dispersion exiting test PC dispersion exiting test PC dispersion'
! pause

end subroutine