Program plydsp_SAFT_driver


!	This is the driver for polydisperse SAFT.  This code calculates the cloud point, i.e.
!	given a temperature and the molecular weight distribution/composition of one phase,
!	the code will go ahead and calculate the pressure and the composition of the 2nd
!	incipient phase.  




!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!
use essentials

implicit none

!	Declaring local (thermo_props) variables

real (KIND=double) :: rhox
real (KIND=double), allocatable, dimension(:) :: m
real (KIND=double), allocatable, dimension(:,:) :: u0
real (KIND=double), allocatable, dimension(:) :: e_ii
real (KIND=double), allocatable, dimension(:,:,:,:) :: kappa
real (KIND=double), allocatable, dimension(:,:) :: sig
real (KIND=double), allocatable, dimension(:) :: v00
real (KIND=double), allocatable, dimension(:,:) :: kij
real (KIND=double), allocatable, dimension(:) :: x
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fa
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fb
real (KIND=double), allocatable, dimension(:) :: MW, x_mass, MW_tmp2
integer, allocatable, dimension(:) :: N_sites
real (KIND=double), allocatable, dimension(:) :: chpt1, chpt2
real (KIND=double) :: Pcalc1, Pcalc2 
real (KIND=double) :: Prsm, Arsm, Srsm, Ursm, Grsm, Hrsm, Pcalc, Gtotal
real (KIND=double), allocatable, dimension(:) :: chpt, fugacity, chpt_res

!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897

!	Declaring local working variables
real (KIND=double) :: den1, den2, rhobest1		!	density guess for phase 1 and 2
integer :: i, j, a, b, k, l, w
character*20 filein, fileout
integer :: ndp				! number of data points 
integer :: polymer						! polymer flag (0) for no and (1) for yes
real (KIND=double) :: sum, sum1, sum2, sum3, sum4, temp
real (KIND=double), allocatable, dimension(:,:) :: molefrac
real (KIND=double), allocatable, dimension(:) :: TK_all
real (KIND=double), allocatable, dimension(:) :: freq_pc
real (KIND=double), allocatable, dimension(:) :: x_unknown, x_known, x_poly_known
real (KIND=double), allocatable, dimension(:) :: x_poly_unknown
real (KIND=double) :: PMn_2
real (KIND=double) :: pd_mfrac_total2, x_unknown_p
real (KIND=double) :: KA, KB
real (KIND=double) :: PMn_unknown
integer :: temp2
!	Declaring variables used by ERSET
integer :: iersvr = 4
integer :: ipact = 0
integer :: isact = 0

!	Declaring variables used by DNEQNF
real (KIND=double) :: errrel, fnorm, fnorm_best
integer :: itmax, variables
real (KIND=double), allocatable, dimension(:) :: xguess, xresult
real (KIND=double), allocatable, dimension(:) :: xbest, xstore

real (KIND=double), dimension(5) :: rparam
integer :: iparam(6), iparam2(6), ldfjac
real (KIND=double), dimension(7) :: rparam2
real (KIND=double), dimension(4) :: xscale, fscale, fvec, xscale2, fscale2
real (KIND=double), dimension(4,4) :: fjac
!integer :: iscale1, iscale2, iscale3, iscale4


external fcn_equil
external PTthermo_props

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!	Calling input_parameters to get SAFT parameters.  these will be temperature and pressure
!		independent parameters of each system.

call input_parameters		! get the Chen & Kreglewski coefficients

write(6,*) 'This is polydisperse Tx SAFT'
write(6,*) ' '

write(6,*) 'Enter the input file name'
read(5,*) filein
write(6,*) 'Enter the output file name'
read(5,*) fileout

open(unit=10, file=filein, status='old')
open(unit=11, file=fileout, status='unknown')

!	Reading input parameters

read(10,*) ncp					! number of real components
allocate(x_unknown(ncp), x_known(ncp), chpt1(ncp), chpt2(ncp), mw_tmp2(ncp))

read(10,*) den1, den2			! density guess (mol/cm3) for liquid and vapor phase


read(10,*) KA, KB				! Ka and Kb guess
read(10,*) disp_flag			! dispersion flag (0) for C&K and (1) for PCSAFT
read(10,*) status11				! association flag (0) for no association and (1) for association
read(10,*) polymer				! polymer flag (0) for mole frac and (1) for mass frac
read(10,*) npc					! number of polydisperse components 
read(10,*) nsc					! number of pseudo components in polydisperse polymer

allocate(freq_pc(nsc), x_poly_known(nsc), x_poly_unknown(nsc))

do i = npc+1, ncp-1
  read(10,*) x_unknown(i)
end do

sum1 = 0.0d0
sum2 = 0.0d0
sum3 = 0.0d0
sum4 = 0.0d0
!	Reading the molecular weight and mass frequency distribution of the specified phase

do i = 1, nsc
  read(10,*) MW_pc(i), freq_pc(i)	! MW of pseudocomponent and mass frequency 
  sum1 = sum1 + freq_pc(i)
  sum2 = sum2 + freq_pc(i) / MW_pc(i)
  sum3 = sum3 + freq_pc(i) * MW_pc(i)
  sum4 = sum4 + freq_pc(i) * MW_pc(i) * MW_pc(i)
end do

if (nsc.gt.0) then
!  PMn_known = sum3/sum1
!  Pmw_known = sum4/sum3
  PMn_known = sum1/sum2
  PMw_known = sum3/sum1
end if

do i = 1, nsc						! convert to mole fraction on a SOLVENT FREE basis
  pd_mfrac_tmp(i) = freq_pc(i)/MW_pc(i)/sum2			! polydisper mole fraction of pseudocomp i
end do


!	Reading the SAFT parameter of the polydisperse component  
do i = 1, npc
  read(10,*) v00_tmp(i), u0_tmp(i,i)
  read(10,*) alpha(i), N_sites_tmp(i)
  read(10,*) e_ii_tmp(i)
end do
MW_tmp(1) = PMn_known


do i = 1, nsc
  m_tmp2(i) = MW_pc(i) * alpha(1)
  v00_tmp2(i) = v00_tmp(1)
  u0_tmp2(i,i) = u0_tmp(1,1)
  N_sites_tmp2(i) = N_sites_tmp(1)
  e_ii_tmp2(i) = e_ii_tmp(1)
end do

!	Reading the SAFT parameters of the solvents
do i = 1+npc, ncp					
  read(10,*) MW_tmp(i)			
  read(10,*) v00_tmp(i), u0_tmp(i,i), m_tmp(i)	
  read(10,*) N_sites_tmp(i)		! number of bonding sites on each component
  read(10,*) e_ii_tmp(i)			
end do


do i = nsc+1, nsc+ncp-1
  v00_tmp2(i) = v00_tmp(i-nsc+npc)
  u0_tmp2(i,i) = u0_tmp(i-nsc+npc,i-nsc+npc)
  N_sites_tmp2(i) = N_sites_tmp(i-nsc+npc)
  e_ii_tmp2(i) = e_ii_tmp(i-nsc+npc)
  m_tmp2(i) = m_tmp(i-nsc+npc)
end do



do i=1, ncp
  do j=1, ncp
    read(10,*) kij_tmp(i,j)		! binary mixture parameter input
  end do
end do

do i = 1, nsc
  do j = 1, nsc
    kij_tmp2(i,j) = 0.0d0
  end do
end do

do i = 2, ncp
  do j = 2, ncp
    kij_tmp2(nsc+i-1, nsc+j-1) = kij_tmp(i,j)
  end do
end do

do j = 2, ncp
  do i = 1, nsc
    kij_tmp2(j+nsc-1, i) = kij_tmp(1,j)
	kij_tmp2(i,j+nsc-1) = kij_tmp(1,j)
  end do
end do


do j = 1, ncp
  do i = 1, N_sites_tmp(j)
    do k=1, ncp
	  do l=1, N_sites_tmp(k)  
		read(10,*) fa_tmp2(j,k,i,l,2), kappa_tmp2(j,k,i,l)	! association energy & volume	  
	  end do
	end do
  end do
end do



read(10,*) variables  ! number of variables used in root finding i.e., P1=P2, chempot1=chempot2..
read(10,*) errrel	  ! relative error to stop DNEQNF
read(10,*) itmax	  ! max number of function evaluations

read(10,*) ndp					! number of trial data points
allocate(molefrac(ndp, ncp), TK_all(ndp), xbest(variables), xstore(variables))

do i = 1, ndp

  read(10,*) TK_all(i)			!  temperature of point i
  sum = 0.0d0
  do j = 1, ncp-1
    read(10,*) molefrac(i,j)	!  the composition of phase I for point i. The first point 
	sum = sum + molefrac(i,j)	!  should be the polydisperse component.
  end do
  molefrac(i,ncp) = 1.0d0 - sum

end do

close(unit=10)



!	Write out inputs
write(11,*) ' '

if (disp_flag.eq.0) write(11,*) 'SAFT code for cloud point calculations'
if (disp_flag.eq.1) write(11,*) 'PCSAFT code for cloud point calculations'
write(11,*) '**************************************'

write(11,*) 'number of components: ', ncp
write(11,*) 'number of polydisperse comp:  ', npc
write(11,*) 'number of pseudocomp in polydisp comp:  ', nsc 
write(11,*) 'polymer flag (0)=no, (1)=yes:  ', polymer
write(11,*) 'association flag:  ', status11
write(11,*) 'dispersion flag:  ', disp_flag
write(11,*) 'number of data points:  ', ndp 

do i = npc+1, ncp-1
  write(11,*) 'guess phase 2 composition from last:', x_unknown(i)
end do

write(11,*) 'density guess for phase1 (mol/cm3)', den1
write(11,*) 'density guess for phase2 (mol/cm3)', den2
write(11,*) 'number avg MW of polydisperse comp: ', PMn_known
write(11,*) 'wt avg MW of polydisperse comp:  ', PMw_known

write(11,*) ' '
write(11,*) 'max number of function evaluation:  ', itmax
write(11,*) 'relative errors allowed:  ', errrel
write(11,*) 'number of eqns solved:  ', variables
write(11,*) ' '

write(11,*) 'MW(i)           frequency(i) '
do i = 1, nsc
  write(11,*) MW_pc(i), freq_pc(i)
end do

do i = 1, npc
  write(11,*) 'component', i, 'parameters'
  write(11,*) 'v00,u0', v00_tmp(i), u0_tmp(i,i)
  write(11,*) 'N_Sites:', N_sites_tmp(i)
  write(11,*) 'e_ii:', e_ii_tmp(i)
end do
do i = 1+npc, ncp
  write(11,*) 'component', i, 'parameters'
  write(11,*) 'v00, u0, m', v00_tmp(i), u0_tmp(i,i), m_tmp(i)
  write(11,*) 'N_sites:', N_sites_tmp(i)
  write(11,*) 'e_ii: ', e_ii_tmp(i)
end do
write(11,*) ' '
write(11,*) 'kij values'
do i =1, ncp
  do j=1, ncp
    write(11,*) kij_tmp(i,j)
  end do
end do
write(11,*) ' '
write(11,*) 'fa and kappa values'
do j = 1, ncp
  do i = 1, N_sites_tmp(j)
    do k=1, ncp
	  do l=1, N_sites_tmp(k)  
		write(11,*) fa_tmp2(j,k,i,l,2), kappa_tmp2(j,k,i,l)	  
	  end do
	end do
  end do
end do
write(11,*) '***********************************'

!	Calculating what needs to be calculated


forall (i=1:ncp)
	sig_tmp(i,i) = (v00_tmp(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:ncp, j=1:ncp, i .ne. j)
	sig_tmp(i,j) = 0.5d0 * (sig_tmp(i,i) + sig_tmp(j,j))
end forall



forall (i=1:nsc+ncp-1)
	sig_tmp2(i,i) = (v00_tmp2(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:nsc+ncp-1, j=1:nsc+ncp-1, i .ne. j)
	sig_tmp2(i,j) = 0.5d0 * (sig_tmp2(i,i) + sig_tmp2(j,j))
end forall

!	converting v00 from mL/mol to angstroms /molecule
forall (i=1:ncp)
	v00_tmp(i) = v00_tmp(i) / AVOG
end forall

forall (i=1: nsc+ncp-1)
	v00_tmp2(i) = v00_tmp2(i)/ AVOG
end forall

den1 = den1 * sig_tmp(1,1)**3 * AVOG
den2 = den2 * sig_tmp(1,1)**3 * AVOG



max = 0
do i = 1, ncp
	if (max .lt. N_sites_tmp(i)) then
	 max = N_sites_tmp(i)
	end if
end do

allocate (m(ncp), u0(ncp,ncp), e_ii(ncp), kappa(ncp,ncp, max, max), sig(ncp, ncp), v00(ncp), &
			& kij(ncp, ncp), x(ncp), fa(ncp, ncp, max, max, 2), fb(ncp, ncp, max, max, 1), &
			& N_sites(ncp), chpt(ncp), xguess(variables), xresult(variables), fugacity(ncp), &
			& chpt_res(ncp))

if (polymer.eq.1) then			! convert mass fraction to mole fraction
  do i = 1, ndp
    sum = 0.0d0
    do j = 1, ncp
	  sum = sum + molefrac(i,j) / MW_tmp(j)
    end do
	
    forall (j=1:ncp)
      molefrac(i,j) =	molefrac(i,j)/MW_tmp(j)/sum
    end forall

	
  end do
end if
!	Starts the real calculations
!***************************************************************************

do w = 1, ndp

!	Calculate mole fraction of the pseudocomponents in the given phase.
  do i = 1, nsc
	x_poly_known(i) = molefrac(w,1)* pd_mfrac_tmp(i)
  end do

write(11,*) 'x_poly_known '
do i = 1, nsc
write(11,*) mw_pc(i), x_poly_known(i)
end do
write(11,*) ' '





!	Calculate mole fraction of the pseudocomponents in the guess phase
  do i = 1, nsc
    x_poly_unknown(i) = x_poly_known(i) * dexp(KA+KB*MW_pc(i))
  end do


!	Calculate the average MW and chain length in guess phase.
!	Setting up the iteration vector
  xguess(1) = den1
  xguess(2) = den2
  xguess(3) = KA
  xguess(4) = KB
  do i = 1, ncp-npc-1 
	xguess(4+i) = x_unknown(i+npc) 
  end do

  TK = TK_all(w)
  sum = 0.0d0
  do i = 1, ncp-1
    x_tmp(i) = molefrac(w,i)
	sum = sum + x_tmp(i)
  end do
  x_tmp(ncp) = 1.0d0 - sum

  do i = 1, ncp
    x_known(i) = x_tmp(i)
  end do


!	assigning association parameters

  do j = 1, ncp
    do i = 1, N_sites_tmp(j)
	  do k = 1, ncp
	    do l = 1, N_sites_tmp(k)
		  fb_tmp(j,k,i,l,1) = fa_tmp2(j,k,i,l,2)/TK/TK * exp(fa_tmp2(j,k,i,l,2)/TK)
		  fa_tmp(j,k,i,l,1) = exp(fa_tmp2(j,k,i,l,2)/TK) - 1.0d0 
		  fb(j,k,i,l,1) = fb_tmp(j,k,i,l,1)
		  kappa_tmp(j,k,i,l) = kappa_tmp2(j,k,i,l)
	    end do
	  end do
    end do
  end do





!	DNEQNF takes the densities and composition of one phase as fixed inputs and calculate the
!	composition of the 2nd phase satisfying chemical potential and pressure equality

	call DN4QBJ(iparam, rparam)
	count = 0
	iparam(3) = 5000
	iparam(4) = 20000
	
	iparam(6) = 1

  do i = 1, variables
    xscale(i) = 1.0d0
	 fscale(i) = 1.0d0
	 fscale2(i) = 1.0d0
	xstore(i) = xguess(i)
  end do

!  iscale1 = -int(log10(abs(xguess(1))))
!  iscale2 = -int(log10(abs(xguess(2))))
!  iscale3 = -int(log10(abs(xguess(3))))
!  iscale4 = -int(log10(abs(xguess(4))))-2
  
!  xguess(1) = xguess(1) * (10.0d0**iscale1)
!  xguess(2) = xguess(2) * (10.0d0**iscale2)
!  xguess(3) = xguess(3) * (10.0d0**iscale3)
!  xguess(4) = xguess(4) * (10.0d0**iscale4)

!	ldfjac = 4
!   iparam2(1) = 0
!  write(11,*) 'in main', iscale1, iscale2, iscale3, iscale4



  fnorm_best = 100.0
  call ERSET(iersvr, ipact, isact)
	ipact = 1
	isact = 0

  do i = 1, 14

    call DNEQNF(fcn_equil, errrel, variables, itmax, xguess, xresult, fnorm)
	do j = 1, variables
	  xguess(j) = xresult(j)
	end do	

	if (fnorm.lt.fnorm_best) then
      do j = 1, variables

        xbest(j) = xresult(j)
	    xguess(j) = xbest(j)
      end do

      fnorm_best = fnorm
	  write(6,*) ' '
	end if


	if (i.eq.15) then 
		write(11,*) '*******************************************'
	  do k=1, variables
	    xguess(i) = xstore(i)
	  end do
	  xguess(3) = xguess(3) + xguess(3) * 0.3
	end if

	if (i.eq.30) then 
			write(11,*) '*******************************************'
	  do k=1, variables
	    xguess(i) = xstore(i)
	  end do
	  xguess(3) = xguess(3) - xguess(3) * 0.3
	end if

	if (i.eq.45) then 
			write(11,*) '*******************************************'
	  do k=1, variables
	    xguess(i) = xstore(i)
	  end do

	  xguess(4) = xguess(4) + xguess(4) * 0.3
	end if

	if (i.eq.60) then 
			write(11,*) '*******************************************'
	  do k=1, variables
	    xguess(i) = xstore(i)
	  end do

	  xguess(4) = xguess(4) - xguess(4) * 0.3
	end if

  end do


!   call DNEQBF(fcn_equil, variables, xguess, xscale, fscale, iparam, rparam, xresult, fvec) 
!   call DUNLSF (fcn_equil, variables, variables, xguess, xscale2, fscale2, iparam2, rparam2, &
!		&		xresult, fvec, fjac, ldfjac)

!  xresult(1)= xresult(1) / 10.0d0**iscale1
!  xresult(2)= xresult(2) / 10.0d0**iscale2
!  xresult(3)= xresult(3) / 10.0d0**iscale3
!  xresult(4)= xresult(4) / 10.0d0**iscale4


  write(6,*) 'count', count
  write(11,*) 'count', count
  write(11,*) 'fnorm', fnorm_best
  write(11,*) ' '
  write(11,*) 'KA', xbest(3)
  write(11,*) 'KB', xbest(4)

  write(6,*) 'hello'
  write(11,*) 'density of fixed phase', xbest(1)/sig_tmp(1,1)**3 / AVOG
  write(11,*) 'density of varied phase', xbest(2)/sig_tmp(1,1)**3 / AVOG

  rhobest1 = xbest(1) 
!	Calculate phase II polymer mole fractions 
  KA = xbest(3) 
  KB = xbest(4)

  x_unknown_p = 0.0d0  
  do i = 1, nsc
    x_poly_unknown(i) = x_poly_known(i) * dexp(KA+KB*MW_pc(i))
    x_unknown_p = x_unknown_p + x_poly_unknown(i)
  end do


  sum = 0.0d0
  sum2 = 0.0d0
  do i = 1, nsc
    sum = sum + x_poly_unknown(i) 
    sum2 = sum2 + x_poly_unknown(i) * MW_pc(i)
  end do

  PMn_unknown = sum2/sum
  x_unknown(1) = x_unknown_p
  do i = 1, ncp-npc-1
	x_unknown(i+1) = xresult(4+i)
  end do

  sum = 0.0d0
  do i = 1, ncp-1
    sum = sum + x_unknown(i)
  end do
  x_unknown(ncp) = 1.0d0 - sum

  do i = 1, ncp
    x_known(i) = x_tmp(i)
  end do
  den1 = xbest(1)
  den2 = xbest(2)




do i = 1, ncp
  temp = m_tmp(i)
  m(i) = temp
  temp = e_ii_tmp(i)
  e_ii(i) = temp
  temp = v00_tmp(i)
  v00(i) = temp
  temp2 = N_sites_tmp(i)
  N_sites(i) = temp2

  do j = 1, ncp
	temp = sig_tmp(i,j)
	sig(i,j) = temp
	temp = u0_tmp(i,j)
	u0(i,j) = temp
	temp = kij_tmp(i,j)
	kij(i,j) = temp
 end do
end do

m(1) = alpha(1)*PMN_known
write(6,*) 'gurgle1'

call PTthermo_props(ncp, max, status11, TK, rhobest1, m, x_known, u0, e_ii, kij, N_sites, fa, &
						& fb, kappa, sig, D, v00, Prsm, Ursm, Arsm, Grsm, Srsm, Hrsm, chpt, &
						& Pcalc, fugacity, chpt_res, Gtotal, disp_flag) 



write(6,*) 'gurgle2'



do i = 1, ncp
  mw_tmp2(i) = mw_tmp(i)
end do
mw_tmp2(1) = PMN_unknown

!	converting the mole fractions to mass fractions
sum = 0.0d0
sum2 = 0.0d0
do i = 1, ncp
  sum = sum + x_known(i) * mw_tmp(i)
  sum2 = sum2 + x_unknown(i) * mw_tmp2(i)
end do

do i = 1, ncp
  x_known(i) = x_known(i)* mw_tmp(i)/sum
  x_unknown(i) = x_unknown(i) * mw_tmp2(i) / sum2
end do


  write(11,*) 'xI in mass frac', x_known
  write(11,*) 'xII in mass frac', x_unknown



!  do i = 1, ncp-npc-1 
!	write(11,*) 'xII, 2nd comp', xresult(4+i) 
!  end do

  write(11,*) 'PMn_unknown', PMn_unknown

!  call final_properties(x_known, x_unknown, den1, den2, chpt1, chpt2, Pcalc1, Pcalc2, PMn_unknown)



  write(11,*) '************************************'
  write(11,*) 'TK', TK
  write(11,*) 'Pcalc', Pcalc
  write(11,*) 'fnorm', fvec
  write(11,*)  ' '     
!  write(11,*) 'Pcal1, Pcalc2', Pcalc1, pcalc2
!  write(11,*) 'chpt1', chpt1(2)
!  write(11,*) 'chpt2', chpt2(2)

end do
END PROGRAM

 











