SUBROUTINE plydsp_zeta_s(sig, u0, rho_spheres, x, x_spheres, sitos1, zeta0, &
	& zeta1, zeta2, zeta3, dz1dt, dz2dt, dz3dt, m, dhs, ncp, TK, rhox, c, dcdt, dddt, &
	& npc, nsc, Pmn, Pchain, alpha, xp, polymerx, polymerMW)

!
!	Purpose:
!		Determination of the zetas used in the hard sphere mixture compressibility 
!			factor Z_hs_mix and the g(r) calculations.  The temperature derivatives
!			of zeta are also calculated.  This particular code assumes that the first
!			component is the polymer and that only one continuous polymer exists in the
!			system.
!
!		zeta(k) = (pi/6)(number of molecules per volume)* sum[x(i)*m(i)*(d(i,i)^k)]
!			where x(i) is the mole fraction of i component, m(i) is the chain length 
!			(number of spheres/molecule) and d(i,i) is the temp. dependent reduced diameter
!
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			8/03/2000		P.D. Ting				Thermoprop subroutines

implicit none
!	Checking single & double precision 

integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)


!	Declare local parameter
real (KIND=double), parameter :: pi = 3.141592654d0
real (KIND=double) :: pi_over6 = pi / 6.0D0



!	Declaring calling variables

integer, intent(in) :: ncp, npc, nsc				! declaring number of components
real (KIND=double), intent(in) :: TK				! temperature in K
real (KIND=double), intent(in) :: rhox				! number of molecules(i.e. chains) per volume (anstroms)
real (KIND=double), intent(in) :: alpha
real (KIND=double), intent(in), dimension(nsc) :: polymerx
real (KIND=double), intent(in), dimension(nsc) ::polymerMW
real (KIND=double), intent(in), dimension(ncp,ncp) :: sig	! temp independent diameter
real (KIND=double), intent(in),  dimension(ncp,ncp) :: u0 
real (KIND=double), intent(in),  dimension(ncp) :: m		! chain length of molecule i
real (KIND=double), intent(in),  dimension(ncp) :: x		! mole fraction of component i in phase j
real (KIND=double), intent(out), dimension(ncp, ncp) :: c	! variable lump used in calculations
real (KIND=double), intent(out), dimension(ncp, ncp) :: dcdt !derivative of c wrt T
real (KIND=double), intent(out) :: zeta0					! zeta0
real (KIND=double), intent(out) :: zeta1					! zeta1
real (KIND=double), intent(out) :: zeta2					! zeta2
real (KIND=double), intent(out) :: zeta3					! zeta3
real (KIND=double), intent(out) :: dz1dt					! derivative of zeta1 wrt T
real (KIND=double), intent(out) :: dz2dt					! derivative of zeta2 wrt T
real (KIND=double), intent(out) :: dz3dt					! derivative of zeta3 wrt T
real (KIND=double), intent(out), dimension(ncp) :: dddt		! derivative of dhs wrt T reduced by sig(1,1)
real (KIND=double), intent(out) :: rho_spheres		! density of spheres (# spheres per volume angstrom)
real (KIND=double), intent(out), dimension(ncp) :: dhs		! temp. dependent diameter
real (KIND=double), intent(out), dimension(ncp) :: sitos1	! dhs(i)/sig(1,1) (reduced temp. dependent sphere diameter)
real (KIND=double), intent(out), dimension(ncp) :: x_spheres ! x(i)*m(i)/ sum[x(i)*m(i)]	
real (KIND=double), intent(out) :: PMn, Pchain
real (KIND=double), intent(out) :: xp

!	Declare local variable
real (KIND=double), dimension(ncp) :: sitos1s			! sitos1 ^ 2
real (KIND=double), dimension(ncp) :: sitos1c			! sitos1 ^ 3	
real :: ratio, sum1								! sum of (x_i * m_i)
integer :: i, j								! index counters


!	Calculate the average molecular weight and average polymer chain length
xp = 0.0d0
sum1 = 0.0d0
do i = 1, nsc
  xp = xp + polymerx(i)
  sum1 = sum1 + polymerx(i) * polymerMW(i)

end do

PMn = sum1 / xp
Pchain = PMn * alpha
m(1) = Pchain


!	Calculate the hard sphere diameter
forall (i=1:ncp)
	dhs(i) = sig(i,i) * (1.0D0 - 0.12D0 * exp(-3.0D0*u0(i,i) / TK))
	dddt(i) = -sig(i,i) * 0.12D0 * exp(-3.0D0 * u0(i,i) / TK) * 3.0D0 * u0(i,i) / TK**2 / sig(1,1)
	sitos1(i) = dhs(i) / sig(1,1)
end forall

!	Calculating c(i,j)
forall (i=1:ncp, j=1:ncp)
	c(i,j) = sitos1(i) * sitos1(j) / (sitos1(i)+sitos1(j))
	dcdt(i,j) = (sitos1(j)**2 * dddt(i) + sitos1(i)**2 * dddt(j))/ (sitos1(i) + sitos1(j))**2
end forall

forall (i=1:ncp)
	sitos1s(i) = sitos1(i) * sitos1(i)
	sitos1c(i) = sitos1s(i) * sitos1(i)
end forall

!	Calculating zeta and derivative of zeta wrt temperature

!	Initializing variables
zeta0 = 0.0d0
zeta1 = 0.0d0
zeta2 = 0.0d0
zeta3 = 0.0d0
dz1dt = 0.0d0
dz2dt = 0.0d0
dz3dt = 0.0d0

do i = npc+1, ncp
  zeta0 = x(i) * m(i) + zeta0
  zeta1 = x(i) * m(i) * sitos1(i) + zeta1
  zeta2 = x(i) * m(i) * sitos1s(i) + zeta2
  zeta3 = x(i) * m(i) * sitos1c(i) + zeta3

  dz1dt = x(i) * m(i) * dddt(i) + dz1dt
  dz2dt = x(i) * m(i) * 2.0d0 * sitos1(i) * dddt(i) + dz2dt	 
  dz3dt = x(i) * m(i) * 3.0d0 * sitos1s(i) * dddt(i) + dz3dt
end do

zeta0 = zeta0 + xp * Pchain
zeta1 = zeta1 + xp * Pchain * sitos1(1)
zeta2 = zeta2 + xp * Pchain * sitos1s(1)
zeta3 = zeta3 + xp * Pchain * sitos1c(1)

dz1dt = dz1dt + xp * Pchain * dddt(1)
dz2dt = dz2dt + xp * Pchain * 2.0d0 * sitos1(1) * dddt(1)
dz3dt = dz3dt + xp * Pchain * 3.0d0 * sitos1s(1) * dddt(1) 


!	Ratio is the ratio of the total number of spherical segments to the total number of 
!		molecules, i.e. sum of x_i * m_i, the effective chain length of the conformal "pure"
!		fluid (mixture)

ratio = zeta0

zeta0 = pi_over6 * rhox * zeta0
zeta1 = pi_over6 * rhox * zeta1
zeta2 = pi_over6 * rhox * zeta2
zeta3 = pi_over6 * rhox * zeta3

dz1dt = pi_over6 * rhox * dz1dt
dz2dt = pi_over6 * rhox * dz2dt
dz3dt = pi_over6 * rhox * dz3dt

!write(6,*) 'zeta 3 is', zeta3

!pause


!	rho_spheres is the total sphere density as opposed to molecule density
rho_spheres = rhox * ratio



!	x_spheres is x(i)*m(i) / sum[x(i)*m(i)]

forall (i=1:ncp)
	x_spheres(i) = x(i) * m(i) / ratio 
end forall

	 
end subroutine
