Program plydsp_SAFT_driver


!	This is the driver for polydisperse SAFT.  This code calculates the cloud point, i.e.
!	given a temperature and the molecular weight distribution/composition of one phase,
!	the code will go ahead and calculate the pressure and the composition of the 2nd
!	incipient phase.  




!	Record of revisions:
!			Date		Programmer				Description of Change
!			====		==========				=====================
!			9/01/00 	P.D. Ting				inception
!
use essentials

implicit none

!	Declaring local (thermo_props) variables

real (KIND=double) :: rhox
real (KIND=double), allocatable, dimension(:) :: m
real (KIND=double), allocatable, dimension(:,:) :: u0
real (KIND=double), allocatable, dimension(:) :: e_ii
real (KIND=double), allocatable, dimension(:,:,:,:) :: kappa
real (KIND=double), allocatable, dimension(:,:) :: sig
real (KIND=double), allocatable, dimension(:) :: v00
real (KIND=double), allocatable, dimension(:,:) :: kij
real (KIND=double), allocatable, dimension(:) :: x
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fa
real (KIND=double), allocatable, dimension(:,:,:,:,:) :: fb
real (KIND=double), allocatable, dimension(:) :: MW, x_mass
integer, allocatable, dimension(:) :: N_sites
real (KIND=double) :: Prsm, Arsm, Srsm, Ursm, Grsm, Hrsm, Pcalc, Gtotal
real (KIND=double), allocatable, dimension(:) :: chpt, fugacity, chpt_res

!	Declaring local parameters
real (KIND=double) :: AVOG = 0.602252d0 			! Nav / (10^8)^3
real (KIND=double) :: R = 83.1451					! cm3 bar/mol /k
real (KIND=double) :: pi = 3.141592654d0
real (KIND=double) :: tau = 0.7404804897

!	Declaring local working variables
real (KIND=double) :: den1, den2		!	density guess for phase 1 and 2
integer :: i, j, a, b, k, l, w
character*20 filein, fileout
integer :: ndp, npc, nsc				! number of data points 
integer :: polymer						! polymer flag (0) for no and (1) for yes
real (KIND=double) :: sum, sum1, sum2, sum3
real (KIND=double), allocatable, dimension(:,:) :: molefrac
real (KIND=double), allocatable, dimension(:) :: TK_all
real (KIND=double), allocatable, dimension(:) :: MW_pc, freq_pc, pd_mfrac
real (KIND=double), allocatable, dimension(:) :: x_unknown, x_known
real (KIND=double), allocatable, dimension(:) :: chpt1, chpt2
real (KIND=double) :: Pcalc1, Pcalc2 
real (KIND=double) :: PMn_2
real (KIND=double) :: pd_mfrac_total2


!	Declaring variables used by ERSET
integer :: iersvr = 4
integer :: ipact = 1
integer :: isact = 0

!	Declaring variables used by DNEQNF
real (KIND=double) :: errrel, fnorm
integer :: itmax, variables
real (KIND=double), allocatable, dimension(:) :: xguess, xresult

external fcn_equil

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!	Calling input_parameters to get SAFT parameters.  these will be temperature and pressure
!		independent parameters of each system.

call input_parameters		! get the Chen & Kreglewski coefficients

write(6,*) 'This is polydisperse Tx SAFT'
write(6,*) ' '

write(6,*) 'Enter the input file name'
read(5,*) filein
write(6,*) 'Enter the output file name'
read(5,*) fileout

open(unit=10, file=filein, status='old')
open(unit=11, file=fileout, status='unknown')

!	Reading input parameters

read(10,*) ncp					! number of real components

allocate(x_unknown(ncp), x_known(ncp), chpt1(ncp), chpt2(ncp))

read(10,*) den1, den2			! density guess (mol/cm3) for liquid and vapor phase
do i = 1, ncp-1
  read(10,*) x_unknown(i)		! unknown phase mole fraction
end do
 
read(10,*) disp_flag			! dispersion flag (0) for C&K and (1) for PCSAFT
read(10,*) status11				! association flag (0) for no association and (1) for association
read(10,*) polymer				! polymer flag (0) for mole frac and (1) for mass frac
read(10,*) npc					! number of polydisperse components 
read(10,*) nsc					! number of pseudo components in polydisperse polymer

allocate(MW_pc(nsc), freq_pc(nsc), pd_mfrac(nsc))

sum1 = 0.0d0
sum2 = 0.0d0
sum3 = 0.0d0

!	Reading the molecular weight and mass frequency distribution of the specified phase

do i = 1, nsc
  read(10,*) MW_pc(i), freq_pc(i)	! MW of pseudocomponent and mass frequency (must sum to 1)
  sum1 = sum1 + freq_pc(i)
  sum2 = sum2 + freq_pc(i) / MW_pc(i)
  sum3 = sum3 + freq_pc(i) * MW_pc(i)
end do

if (nsc.gt.0) then
  PMn = sum1/sum2
  PMw = sum3/sum1
end if

do i = 1, nsc						! convert to mole fraction on a SOLVENT FREE basis
  pd_mfrac(i) = freq_pc(i)/MW_pc(i)/sum2			! polydisper mole fraction of pseudocomp i
end do


!	Reading the initial guess for the num. avg. MW of phase beta polydisperse comp and xp
if (nsc.ne.0) then
  read(10,*) PMn_2
  read(10,*) pd_mfrac_total2
end if

!	Reading the SAFT parameter of the polydisperse component  
do i = 1, npc
  read(10,*) v00_tmp(i), u0_tmp(i,i)
  read(10,*) alpha(i), N_sites_tmp(i)
  read(10,*) e_ii_tmp(i)
end do


!	Reading the SAFT parameters of the solvents
do i = 1+npc, ncp					
read(10,*) MW_tmp(i)			
read(10,*) v00_tmp(i), u0_tmp(i,i), m_tmp(i)	
read(10,*) N_sites_tmp(i)		! number of bonding sites on each component
read(10,*) e_ii_tmp(i)			
end do



do i=1, ncp
  do j=1, ncp
    read(10,*) kij_tmp(i,j)		! binary mixture parameter input
  end do
end do

do j = 1, ncp
  do i = 1, N_sites_tmp(j)
    do k=1, ncp
	  do l=1, N_sites_tmp(k)  
		read(10,*) fa_tmp2(j,k,i,l,2), kappa_tmp2(j,k,i,l)	! association energy & volume	  
	  end do
	end do
  end do
end do


read(10,*) variables  ! number of variables used in root finding i.e., P1=P2, chempot1=chempot2..
read(10,*) errrel	  ! relative error to stop DNEQNF
read(10,*) itmax	  ! max number of function evaluations

read(10,*) ndp					! number of trial data points

allocate(molefrac(ndp, ncp), TK_all(ndp))

do i = 1, ndp

  read(10,*) TK_all(i)			!  temperature of point i
  sum = 0.0d0
  do j = 1, ncp-1
    read(10,*) molefrac(i,j)	!  the composition of phase I for point i
	sum = sum + molefrac(i,j)
  end do
  molefrac(i,ncp) = 1.0d0 - sum

end do

close(unit=10)



!	Write out inputs
write(11,*) ' '

if (disp_flag.eq.0) write(11,*) 'SAFT code for cloud point calculations'
if (disp_flag.eq.1) write(11,*) 'PCSAFT code for cloud point calculations'
write(11,*) '**************************************'

write(11,*) 'number of components: ', ncp
write(11,*) 'number of polydisperse comp:  ', npc
write(11,*) 'number of pseudocomp in polydisp comp:  ', nsc 
write(11,*) 'polymer flag (0)=no, (1)=yes:  ', polymer
write(11,*) 'association flag:  ', status11
write(11,*) 'dispersion flag:  ', disp_flag
write(11,*) 'number of data points:  ', ndp 
write(11,*) 'density guess for phase1 (mol/cm3)', den1
write(11,*) 'density guess for phase2 (mol/cm3)', den2
write(11,*) 'number avg MW of polydisperse comp: ', PMn
write(11,*) 'wt avg MW of polydisperse comp:  ', PMw

write(11,*) ' '
write(11,*) 'max number of function evaluation:  ', itmax
write(11,*) 'relative errors allowed:  ', errrel
write(11,*) 'number of eqns solved:  ', variables
write(11,*) ' '

do i = 1, npc
  write(11,*) 'MW(i)           frequency(i) '
  write(11,*) MW_pc(i), freq_pc(i)
end do


do i = 1+npc, ncp
  write(11,*) 'component', i, 'parameters'
  write(11,*) 'v00, u0, m', v00_tmp(i), u0_tmp(i,i), m_tmp(i)
  write(11,*) 'N_sites:', N_sites_tmp(i)
  write(11,*) 'e_ii: ', e_ii_tmp(i)
end do
write(11,*) ' '
write(11,*) 'kij values'
do i =1, ncp
  do j=1, ncp
    write(11,*) kij_tmp(i,j)
  end do
end do
write(11,*) ' '
write(11,*) 'fa and kappa values'
do j = 1, ncp
  do i = 1, N_sites_tmp(j)
    do k=1, ncp
	  do l=1, N_sites_tmp(k)  
		write(11,*) fa_tmp2(j,k,i,l,2), kappa_tmp2(j,k,i,l)	  
	  end do
	end do
  end do
end do

!	Calculating what needs to be calculated


forall (i=1:ncp)
	sig_tmp(i,i) = (v00_tmp(i)*6.0d0*tau/pi/AVOG)**(1.0d0/3.0d0)
end forall

forall (i=1:ncp, j=1:ncp, i .ne. j)
	sig_tmp(i,j) = 0.5d0 * (sig_tmp(i,i) + sig_tmp(j,j))
end forall

!	converting v00 from mL/mol to angstroms /molecule
forall (i=1:ncp)
	v00_tmp(i) = v00_tmp(i) / AVOG
end forall

den1 = den1 * sig_tmp(1,1)**3 * AVOG
den2 = den2 * sig_tmp(1,1)**3 * AVOG



max = 0
do i = 1, ncp
	if (max .lt. N_sites_tmp(i)) then
	 max = N_sites_tmp(i)
	end if
end do

allocate (m(ncp), u0(ncp,ncp), e_ii(ncp), kappa(ncp,ncp, max, max), sig(ncp, ncp), v00(ncp), &
			& kij(ncp, ncp), x(ncp), fa(ncp, ncp, max, max, 2), fb(ncp, ncp, max, max, 1), &
			& N_sites(ncp), chpt(ncp), xguess(variables), xresult(variables))


if (polymer.eq.1) then			! convert mass fraction to mole fraction
  do i = 1, ndp
    sum = 0.0d0
    do j = 1, ncp
	  sum = sum + molefrac(i,j) / MW_tmp(j)
    end do
	
    forall (j=1:ncp)
      molefrac(i,j) =	molefrac(i,j)/MW_tmp(j)/sum
    end forall
  end do
end if

!	Starts the real calculations
!***************************************************************************

do w = 1, ndp

  if (w.ge.2) then
  end if


  TK = TK_all(w)
  do i = 1, ncp
    x_tmp(i) = molefrac(w,i)
  end do

!	assigning association parameters

  do j = 1, ncp
    do i = 1, N_sites_tmp(j)
	  do k = 1, ncp
	    do l = 1, N_sites_tmp(k)
		  fb_tmp(j,k,i,l,1) = fa_tmp2(j,k,i,l,2)/TK/TK * exp(fa_tmp2(j,k,i,l,2)/TK)
		  fa_tmp(j,k,i,l,1) = exp(fa_tmp2(j,k,i,l,2)/TK) - 1.0d0 
		  fb(j,k,i,l,1) = fb_tmp(j,k,i,l,1)
		  kappa_tmp(j,k,i,l) = kappa_tmp2(j,k,i,l)
	    end do
	  end do
    end do
  end do


!	setting the vector that will be varied in the minimization routine.  NOTE THAT WE ARE
!	SET UP TO DO ONLY TWO PHASE CALCULATIONS

  do i = 1, ncp
    x_known(i) = x_tmp(i)
  end do

  xguess(1) = den1
  xguess(2) = den2

  do i = 3,ncp+2-1
    xguess(i) = x_unknown(i-2)
  end do

!  xguess(3) = PMn_2
!  xguess(4) = pd_mfrac_total2


!	DNEQNF takes the densities and composition of one phase as fixed inputs and calculate the
!	composition of the 2nd phase satisfying chemical potential and pressure equality

  call ERSET(iersvr, ipact, isact)
  call DNEQNF(fcn_equil, errrel, variables, itmax, xguess, xresult, fnorm)




  sum = 0.0d0
  do i = 1, ncp-1
    x_unknown(i) = xresult(i+2)
	sum = sum + x_unknown(i)
  end do

  x_unknown(ncp) = 1.0d0 - sum

  call final_properties(x_known, x_unknown, xresult(1), xresult(2), chpt1, chpt2, Pcalc1, Pcalc2)



  write(11,*) '************************************'
  write(11,*) 'TK', TK
  write(11,*) 'density of fixed phase', xresult(1)/sig_tmp(1,1)**3 / AVOG
  write(11,*) 'density of varied phase', xresult(2)/sig_tmp(1,1)**3 / AVOG
  write(11,*) 'composition of known phase', x_tmp
  write(11,*) 'composition of unknown phase', x_unknown
  write(11,*) 'fnorm', fnorm
  write(11,*)  ' '     
  write(11,*) 'Pcal1, Pcalc2', Pcalc1, pcalc2
  write(11,*) 'chpt1', chpt1
  write(11,*) 'chpt2', chpt2

end do
END PROGRAM

 




















