! ****************************************************************
!	Note fb and fa(i,j,a,b,1) have not been calculated 
! ****************************************************************


subroutine test_Q_sub(rhox, ncp, N_sites, Q, dQdT, dQ_drho, dQdrhoi, max, g, dgdrho,&
				& dgdt, rhodgdrho, sig, fa, kappa, fb)

!	subroutine Q_sub
!
!	Program written by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Calculates the strength of bonding function (delta) and all derivatives 
!		used in hydrogen bonding calculations of Wertheim's theory.  This code 
!		assumes that only single bonds are formed between molecules.  For additional
!		information see W.G. Chapman's thesis, Cornell University, 1988.    
!
!	This code calculates:
!		Q, the interaction integral of component i with component j at site k of 
!			component i and site l of component j (called 'delta' in literature).
!		dQdbeta, d(Q)/d(1/kT) 
!		dQdrho, d(Q)/d(rho(m))
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/28/99		P.D. Ting				revised for multicomponent and updated 
!													for fortran 90/95	


 
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp								! number of components
integer, intent(in) :: max								! max # sites on all molecules
real (KIND=double), intent(in) :: rhox					! molecular (chain) density 
integer, intent(in), dimension(ncp) :: N_sites	! # of bonding sites 
real (KIND=double), intent(in), dimension(ncp, ncp) :: g		! distribution function g(i,j)
real (KIND=double), intent(in), dimension(ncp, ncp, ncp) :: dgdrho		! dg(i,j)/d[(rho(m)]
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig		! temp. independent diameter 
real (KIND=double), intent(in), dimension(ncp, ncp, max, max, 2) :: fa	! fa (epsilon AiBj)
real (KIND=double), intent(in), dimension(ncp, ncp, max, max) :: kappa	! kappa AiBj
real (KIND=double), intent(in), dimension(ncp, ncp, max, max, 1) :: fb	! d(fa)/dT
real (KIND=double), intent(in), dimension(ncp, ncp) :: dgdt				! d[g(i,j)]/dT
real (KIND=double), intent(in), dimension(ncp, ncp) :: rhodgdrho		! rho * dg(i,j)/d(rho)
real (KIND=double), intent(out), dimension(ncp, ncp, max, max) :: Q		! association strength
real (KIND=double), intent(out), dimension(ncp, ncp, max, max) :: dQdT	! dQ / dT 
real (KIND=double), intent(out), dimension(ncp, ncp, max, max) :: dQ_drho ! dQ/d(rho)
real (KIND=double), intent(out), dimension(ncp, ncp, max, max, ncp) :: dQdrhoi ! dQ/d[rho(i)]



!	Declaring local parameters
real (KIND=double), parameter :: pi = 3.141592654d0

!	Declaring local variables
integer :: i, j, a, b, k						! index counters
real (KIND=double) :: s, s2, s3					! sig(i,j)/sig(1,1) dummies



!	Here's the meat

!	Initializing variables
do i = 1, ncp
	do a = 1, N_sites(i)
		do j = 1, ncp
			do b = 1, N_sites(b)
				Q(i,j,a,b) = 0.0d0 
				dQdT(i,j,a,b) = 0.0d0
				dQ_drho(i,j,a,b) = 0.0d0
				
				do k = 1, ncp
					dQdrhoi(i,j,a,b,k) = 0.0d0 
				end do
			end do
		end do
	end do
end do



!	Note an unresolved issue is that kappa, the bonding volume, is here assigned to be
!	temperature independent.  However, the sphere volume depends on T.  So there's some
!	unresolved "discrepancy" with this method.  
!	Also note that only one of the spheres in each chain has association sites.


do i = 1, ncp
	do a = 1, N_sites(i)
		do j = 1, ncp
			
			s = sig(i,j) / sig(1,1)
			s2 = s * s
			s3 = s * s2

			do b = 1, N_sites(j)
				Q(i,j,a,b) = g(i,j) * fa(i,j,a,b,1) * kappa(i,j,a,b) * s3 

				dQdT(i,j,a,b) = s3 * kappa(i,j,a,b) * ( fa(i,j,a,b,1) * dgdt(i,j) &
								& - g(i,j) * fb(i,j,a,b,1))

				dQ_drho(i,j,a,b) = s3 * kappa(i,j,a,b) * fa(i,j,a,b,1) * rhodgdrho(i,j)/rhox

				do k = 1, ncp

					dQdrhoi(i,j,a,b,k) = s3 * kappa(i,j,a,b) * fa(i,j,a,b,1) * dgdrho(i,j,k)


				end do
			end do
		end do
	end do
end do 

end subroutine

