subroutine test_dispersion(D, ncp, e_ii, TK, u0, sitos1, kij, x, m, rhox, zeta3, Amf, Pmf, Gmf, &
						& Hmf, Smf, Umf, chptlmf, dddt, dz3dt)

!	Subroutine dispersion
!
!	Program written by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Calculate the dispersion (mean field) contribution of the hard sphere mixtures.
!		This is designated as part of the segment contribution terms (dispersion + HS)
!		We will use vdW-One conformal fluid to deal with mixture effects.  The dispersion 
!		will be calculated using fitted parameters by Chen and Kreglewski (1977).  Parameters
!		determined by Cotterman et al. (1986) can also be used, as in the case of the original
!		SAFT paper.  
!
!		This code calculates:
!				residual mean field pressure [p/rho kT]
!				residual mean field internal energy [U/NkT]
!				residual mean field Helmholtz free energy [A/NkT]
!				residual mean field Gibbs free energy [G/NkT]
!				residual mean field entropy [S/Nk]
!				residual mean field enthalpy [H/NkT]
!				residual chemical mean field potential of components [chptl/kT]
!			
!			where N is the number of CHAINS
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/27/99		P.D. Ting				revised for multicomponent and updated 
!													for fortran 90/95	


 
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp									! number of components
real (KIND=double), intent(in), dimension(4,9) :: D			! d(i,j) parameter from Chen & Kreglewski
real (KIND=double), intent(in), dimension(ncp) :: e_ii		! dispersion e_ii
real (KIND=double), intent(in), dimension(ncp, ncp) :: u0	! (u0_i,i)/k in K 
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! sig(1,1) reduced temp. dependent HS diameter
real (KIND=double), intent(in), dimension(ncp, ncp) :: kij	! k(i,j)
real (KIND=double), intent(in), dimension(ncp) :: x			! mole fraction of component i
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp) :: dddt		! derivative of d(hard sphere) wrt T
real (KIND=double), intent(in) :: TK						! Temperature in Kelvin
real (KIND=double), intent(in) :: rhox						! # molecules per volume
real (KIND=double), intent(in) :: dz3dt						! d(zeta3)/dT
real (KIND=double), intent(in) :: zeta3						! zeta3
real (KIND=double), intent(out) :: Amf						! mean field Helmholtz free energy
real (KIND=double), intent(out) :: Pmf						! mean field pressure
real (KIND=double), intent(out) :: Gmf						! mean field Gibbs free energy
real (KIND=double), intent(out) :: Hmf						! mean field Helmholtz free energy
real (KIND=double), intent(out) :: Smf						! mean field entropy
real (KIND=double), intent(out) :: Umf						! mean field internal energy
real (KIND=double), intent(out), dimension(ncp) :: chptlmf	! mean field res. chemical potential


!	Declaring local parameters
real (KIND=double), parameter :: tau = 0.7404804897			! pi/6*sqrt(2)
real (KIND=double), parameter :: pi = 3.141592654d0

!	Declaring local variables
real (KIND=double), dimension(ncp, ncp) :: u				! temp. dependent u0(i,i)
real (KIND=double), dimension(ncp, ncp) :: v0				! temp. dependent v00(i)
real (KIND=double) :: sum1, sum2, sum3, sum4, sum5
real (KIND=double), dimension(ncp) :: sum7, sum8
real (KIND=double), dimension(ncp) :: du_ktdn				! d(u/kT) / dn
real (KIND=double), dimension(ncp, ncp) :: duiidT			! du(i,i)/dT
real (KIND=double), dimension(ncp) :: dv0idT				! dv0(i)/dT
real (KIND=double), dimension(ncp, ncp) :: duij_kTdT		! d [u(i,j)/kT] / dT
real (KIND=double), dimension(ncp, ncp) :: dv0ijdT			! dv0(i,j) / dT
real (KIND=double) :: du_ktdT								! d (U/kT) / dT
real (KIND=double) :: dAmfdT								! d Amf / dT
integer :: i,j, l												! index counters
real (KIND=double) :: numerator, denominator
real (KIND=double) :: u_kT									! vdW-1 u/kT
real (KIND=double) :: m_sum									! sum[m(i)*x(i)]

!	Meat starts here

!	Initializing the free energies
Amf = 0.0d0
Pmf = 0.0d0
Gmf = 0.0d0

!	Calculating temperature dependent segment dispersion interaction energy u/k (K)
!	and v0.  Note that the v00 differs from the input in that the input is in mL/mol while
!	the v00 related here is converted to ang^3 per molecule.  The v0 values are further made
!	dimensionless with sig(1,1) as it is calculated from reduced temp. dependent HS diameter.

forall (i=1:ncp)
	u(i,i) = u0(i,i) * (1.0d0 + e_ii(i)/TK)
	v0(i,i) = pi * sitos1(i)**3 / 6.0d0 / tau 
end forall


!	Make the mixture to a vdW-1 conformal solution 

forall (i=1:ncp, j=1:ncp, i .ne. j)
	u(i,j) = ( 1.0d0 - kij(i,j) ) * sqrt(u(i,i) * u(j,j))
	v0(i,j) = ( 0.5d0 * (v0(i,i)**(1.0d0/3.0d0) + v0(j,j)**(1.0d0/3.0d0) ) )**3
end forall

!	Make u(i,j) dimensionless by reducing it with temperature

forall (i=1:ncp, j=1:ncp)
	u(i,j) = u(i,j) / TK
end forall

!	Calculating u/kT with the vdW-1 mixing rule

numerator = 0.0d0
denominator = 0.0d0

do i = 1,ncp
	do j = 1, ncp
		
		numerator = numerator + x(i)*x(j)*m(i)*m(j)*u(i,j)*v0(i,j)
		denominator = denominator + x(i)*x(j)*m(i)*m(j)*v0(i,j)
	end do
end do

u_kT = numerator / denominator

!	Calculating effective chain length

m_sum = 0.0d0

do i = 1, ncp
	m_sum = m_sum + x(i) * m(i)
end do


!	Calculating d(U/kT)d(rho(i))
do i = 1, ncp
	sum7(i) = 0.0d0
	sum8(i) = 0.0d0

	do j = 1, ncp
		sum7(i) = sum7(i) + 2.0d0*m(i)*x(j)*m(j)*u(i,j)*v0(i,j)
		sum8(i) = sum8(i) + 2.0d0*m(i)*m(j)*x(j)*v0(i,j)
	end do

	du_ktdn(i) = sum7(i) / (rhox * denominator) - numerator*sum8(i)/rhox/denominator**2

end do

!	Calculating d(U(i,i)/kT)/dT and d(v0(i))/dT

forall (i=1:ncp)
	duiidT(i,i) = -u0(i,i) * e_ii(i) / TK / TK
	dv0idT(i) = 3 / sqrt(2.0d0) * sitos1(i) * sitos1(i) * dddt(i) 
end forall

!	Calculating d(U(i,j)/kT)/dT and d(v0(i,j))dT

forall (i=1:ncp, j=1:ncp)
!		First equation on page 17 of Dispersion Derivations 
		duij_kTdT(i,j) = -u(i,j)/ TK / TK + 0.5d0 * (1.0d0 - kij(i,j)) &
						& / sqrt(u(i,i)*u(j,j)) * ( u(j,j) * duiidT(i,i) &
						& + u(i,i) * duiidT(j,j) )

!	Second equation on page 13 of Dispersion Derivations
		dv0ijdT(i,j) =  ( v0(i,i)**(1.0d0/3.0d0) + v0(j,j)**(1.0d0/3.0d0) )**2 / 8.0d0 &
						& * ( dv0idT(i) / (v0(i,i)**(2.0d0/3.0d0)) + dv0idT(j) &
						& / ( v0(j,j)**(2.0d0/3.0d0) )  )
end forall


!	Calculate d(U/kt)/dT at constant V, N.  Page 9 of Dispersion Derivations
sum1 = 0.0d0
sum2 = 0.0d0
sum3 = 0.0d0
sum4 = 0.0d0
sum5 = 0.0d0

do i = 1, ncp
	do j = 1, ncp
		sum1 = sum1 + x(i)*x(j)*m(i)*m(j) * (v0(i,j) * duij_kTdT(i,j) + u(i,j) * dv0ijdT(i,j) )

		sum2 = sum2 + x(i) * x(j) * m(i) * m(j) * v0(i,j)

		sum3 = sum3 + x(i) * x(j) * m(i) * m(j) * u(i,j) * v0(i,j) 

		sum4 = sum4 + x(i) * x(j) * m(i) * m(j) * v0(i,j)

		sum5 = sum5 + x(i) * x(j) * m(i) * m(j) * dv0ijdT(i,j)
	end do
end do

du_ktdT = sum1 / sum2 - sum3 / sum4 / sum4 * sum5




!	Calculating Helmholtz free energy and Pressure
do i = 1,4
	do j = 1, 9
		
		Amf = Amf + m_sum * D(i,j) * u_kT**i * (zeta3 / tau)**j
		Pmf = Pmf + m_sum * D(i,j) * u_kT**i * j * (zeta3/tau)**j
	end do
end do


!	Calculate dA/dT.  See page 5 of Dispersion Derivations

dAmfdT = 0.0d0

do i = 1, ncp
	do j = 1, ncp
		dAmfdT = dAmfdT + D(i,j)*(i * u_kT** (i-1) * du_ktdT * (zeta3/tau)** j &
				& + u_kT**i * j * (zeta3/tau)** j / zeta3 * dz3dt)
	end do
end do

dAmfdT = Amf + TK * m_sum * dAmfdT

Smf = -dAmfdT
Umf = Amf + Smf
Hmf = Umf + Pmf
Gmf = Amf + Pmf

!	Calculating chemical potential

do i = 1, ncp
	chptlmf(i) = 0.0d0 
	do j = 1, 4
		do l = 1,9

			chptlmf(i) = chptlmf(i) + m(i) * D(j,l) * u_kT**j * (zeta3/tau)**l &
						& + rhox * m_sum * D(j,l) * ( j * u_kT**(j-1) * du_ktdn(i) * (zeta3/tau)**l &
						& + u_kT**j * l * (zeta3/tau)**(l-1) / tau * pi / 6.0d0 * m(i) * sitos1(i)**3 )
		end do
	end do
end do

!	Gmf is also 
!	Gmf = 0.0d0
!	do i = 1, ncp
!		Gmf = Gmf + x(i) * chptlmf(i)
!	end do


end subroutine