subroutine test_hard_chain_eos(rhox, x, m, ncp, TK, sitos1, g, chpthc, Phc, Ahc, Ghc, Uhc, Shc, &
						& Hhc, rhodgdrho, dgdrho, dgdt)

!	Subroutine hard_chain_eos
!
!	Program written by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Calculates the thermodynamic properties of a mixture due to hard chains formation.  
!		Each chain i is homonuclear and can have any number of spheres m(i).  
!		The equation is based on Wertheim's bonding theory and on Carnahan-Starling
!		EOS.  Info on the hard spheres (Carnahan-Starling) can be found in Chapter 9 of
!		Applied Statistical Mechanics, by T.M. Reed and K.E. Gubbins
!
!	This code calculates:
!		residual hard chain pressure [p/rho kT]
!		residual internal energy  [U/NkT]
!		residual Helmholtz free energy  [A/NkT]
!		residual Gibbs free energy  [G/NkT]
!		residual Entropy [S/Nk]
!		residual enthalpy  [H/NkT]
!		residual chemical potential of components due to hard chain formation [chptl/kT]
!			N is the number of CHAINS
!			rho is the density of CHAINS (and not spheres)
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/28/99		P.D. Ting				revised for multicomponent and updated 
!													for fortran 90/95	

implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp
real (KIND=double), intent(in), dimension (ncp) :: sitos1	! dhs(i)/sig(1,1) (reduced temp. dependent sphere dia.)
real (KIND=double), intent(in), dimension (ncp) :: m		! chain length of molecule i
real (KIND=double), intent(in), dimension (ncp) :: x		! mole fraction of component i
real (KIND=double), intent(in), dimension (ncp, ncp) :: g	! hard sphere g(r) at contact 
real (KIND=double), intent(in), dimension (ncp, ncp) :: rhodgdrho	! rho*[dg(i,j)/drho]
real (KIND=double), intent(in), dimension (ncp, ncp) :: dgdt		! dg(i,j)/dT		
real (KIND=double), intent(in), dimension (ncp, ncp, ncp) :: dgdrho	! dg(i,j)/drho(k)]
real (KIND=double), intent(in) :: rhox						! chain density
real (KIND=double), intent(in) :: TK						! Temperature (in Kelvin)
real (KIND=double), intent(out) :: Phc						! residual pressure 
real (KIND=double), intent(out) :: Ahc						! residual Helmholtz free energy
real (KIND=double), intent(out) :: Ghc						! residual Gibbs free energy
real (KIND=double), intent(out) :: Uhc						! residual internal energy
real (KIND=double), intent(out) :: Shc						! residual entropy 
real (KIND=double), intent(out) :: Hhc						! residual enthalpy
real (KIND=double), intent(out), dimension (ncp) :: chpthc	! residual chemical potential

!	Declaring local parameters



!	Declaring local variables
real (KIND=double) :: sum1									! temporary summation
real (KIND=double) :: sum2									!  "           "
integer :: i,j												! index counters
real (KIND=double) :: dAhcdT								! d(Ahc)/dT





!	Here's the meat
Phc = 0.0d0
Ahc = 0.0d0

!	Calculate pressure & residual Helmholtz free energy (see pg. 1 in Chain derivations)
do i = 1, ncp
	Phc = Phc + x(i) * (1.0d0 - m(i)) * rhodgdrho(i,i) / g(i,i)
	Ahc = Ahc + x(i) * (1.0d0 - m(i)) * log(g(i,i))
end do

!	Calculate chemical potential (see pg. 1 in Chain derivations)
do i = 1, ncp
	sum1 = 0.0d0

	do j = 1, ncp 
		sum1 = sum1 + x(j) * (1.0d0 -m(j)) * rhox * dgdrho(j,j,i) / g(j,j)
	end do

	chpthc(i) = (1.0d0 - m(i)) * log(g(i,i)) + sum1

end do

!	Calculating d(Ahc)/dT

sum2 = 0.0d0 
do i = 1, ncp
	sum2 = sum2 + x(i) * (1.0d0 - m(i))*dgdt(i,i) / g(i,i)
end do

dAhcdT = Ahc + TK * sum2 


!	Calculating other properties
Ghc = Ahc + Phc 
Shc = -dAhcdT
Uhc = Ahc + Shc
Hhc = Phc + Uhc 

end subroutine
