subroutine test_hbond_thermo(ncp, max, rhox, N_sites, g, dgdrho, sig, fa, kappa, fb, dgdt, &
						& rhodgdrho, Phb, Uhb, Ahb, Ghb, Shb, Hhb, chptlhb, x, TK)

!	Subroutine hbond_thermo
!
!	Program by P. David Ting from codes by Walter G. Chapman
!
!	Purpose:
!		Calculates the changes in thermodynamic properties of a mixture due to association.
!		The theory is due to Wertheim.  This subroutine assumes that only single bonds are 
!		formed between molecules.  For additional information see W.G. Chapman's thesis, 
!		Cornell University, 1988.  
!
!	This code calculates:
!		H-bond contribution to pressure [P/rho kT]
!		H-bond contribution to internal energy [U/NkT]
!		H-bond contribution to Helmholtz free energy [A/NkT]
!		H-bond contribution to Gibbs free energy [G/NkT]
!		H-bond contribution to entropy [S/Nk]
!		H-bond contribution to enthalpy [H/NkT]
!		H-bond contribution to residual chemical potential of component i [chptl/kT]
!
!
!	This routine needs the subroutine Q_sub 
!
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/28/99		P.D. Ting				Revised for multicomponent and updated 
!													for fortran 90/95	


 
implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp												! number of components
integer, intent(in) :: max												! max # sites on all molecules
real (KIND=double), intent(in) :: rhox									! molecular (chain) density
real (KIND=double), intent(in) :: TK									! Temperature in Kelvin
integer, intent(in), dimension(ncp) :: N_sites				! # of bonding sites
real (KIND=double), intent(in), dimension(ncp) :: x						! mole fraction of i
real (KIND=double), intent(in), dimension(ncp, ncp) :: g				! distribution function g(i,j)
real (KIND=double), intent(in), dimension(ncp, ncp, ncp) :: dgdrho		! dg(i,j)/d[rho(m)]
real (KIND=double), intent(in), dimension(ncp, ncp) :: sig				! temp. independent diameter
real (KIND=double), intent(in), dimension(ncp, ncp, max, max, 2) :: fa	! fa (epsilon AiBj)
real (KIND=double), intent(in), dimension(ncp, ncp, max, max) :: kappa	! kappa AiBj
real (KIND=double), intent(in), dimension(ncp, ncp, max, max, 1) :: fb	! d(fa) / d T
real (KIND=double), intent(in), dimension(ncp, ncp) :: dgdt				! d[g(i,j)] / dT
real (KIND=double), intent(in), dimension(ncp, ncp) :: rhodgdrho		! rho * d[g(i,j)] / d(rho)
real (KIND=double), intent(out) :: Phb									! H-Bond contribution to Pressure
real (KIND=double), intent(out) :: Uhb									! H-Bond contribution to U
real (KIND=double), intent(out) :: Ahb									! H-Bond contribution to A
real (KIND=double), intent(out) :: Ghb									! H-Bond contribution to G
real (KIND=double), intent(out) :: Shb									! H-Bond contribution to S
real (KIND=double), intent(out) :: Hhb									! H-Bond contribution to H
real (KIND=double), intent(out), dimension(ncp) :: chptlhb				! H-Bond contribution to res. chem. pot.


!	Declaring local parameters
real (KIND=double), parameter :: pi = 3.141592654d0
 


!	Declaring local variables
real (KIND=double), dimension(ncp, ncp, max, max) :: Q					! association strength
real (KIND=double), dimension(ncp, ncp, max, max) :: dQdT				! dQdT
real (KIND=double), dimension(ncp, ncp, max, max) :: dQ_drho			! dQ / d(rho)
real (KIND=double), dimension(ncp, ncp, max, max, ncp) :: dQdrhoi		! dQ / d[rho(i)]
real (KIND=double), dimension(max, ncp) :: ss							! Xai, mole fraction i not bonded at A
real (KIND=double), dimension(max, ncp) :: ss_new						! next iteration ss
real (KIND=double), dimension(max, ncp) :: ss_new2						! temporary storage for ss_new
real (KIND=double), dimension(max, ncp) :: dss_dbeta					! d(Xai)/d(beta)
real (KIND=double), dimension(max, ncp) :: dss_dbeta_new				! next iteration dss_dbeta
real (KIND=double), dimension(max, ncp) :: dss_dbeta_new2				! temporary storage for dss_dbeta_new
real (KIND=double), dimension(max, ncp, ncp) :: dss_drhoi				! d(Xai) / d[rho(i)]
real (KIND=double), dimension(max, ncp, ncp) :: dss_drhoi_new			! next iteration dss_drhoi
real (KIND=double), dimension(max, ncp, ncp) :: dss_drhoi_new2			! temporary storage for dss_drhoi_new
real (KIND=double), dimension(max, ncp) :: dssdrho						! d(Xai)/ d(rho)
real (KIND=double), dimension(max, ncp) :: dssdrho_new					! next iteration dssdrho
real (KIND=double), dimension(max, ncp) :: dssdrho_new2					! temporary storage for dssdrho_new
real (KIND=double), dimension(ncp) :: ssprod							! product of all Xai for given i
real (KIND=double), dimension(ncp) :: dss_dbetaprod						! product of all dssdbeta for given i
real (KIND=double), dimension(ncp, ncp) :: dss_drhoiprod				! product of all dssdrhoi for given i 
real (KIND=double), dimension(ncp) :: dssdrhoprod						! product of all dssdrho for given i 
real (KIND=single) :: amix												! acceptance ratio
real (KIND=double) :: tol												! tolerance
integer :: max_eval														! max # of iterations
integer :: a,b, i, j, k, attempts										! index counters
real (KIND=double) :: sum, sum1, sum_mu, sum_mu1, sum_mu2
real (KIND=double) :: sum_error											! sum of absolute errors
real (KIND=double) :: prod, prod2, prod3, prod4								! temporary storage 
real (KINd=double) :: sum_deltaA1, sum_deltaU1, sum_deltaP1
!	Here's the meat

!	Calls the subroutine that calculates the association strength

call test_Q_sub(rhox, ncp, N_sites, Q, dQdT, dQ_drho, dQdrhoi, max, g, dgdrho, &
			& dgdt, rhodgdrho, sig, fa, kappa, fb)


!	To find Xai (mole fraction of i not bonded at site a), we need to iterate.  We will specify
!	an acceptance ratio, a tolerance, and a maximum iteration number
amix  = 0.7
tol = 1.0d-8
max_eval = 5000
sum_error = 100.0d0 
attempts = 1



!	We will call ss(i,a) what is termed Xai, the mole fraction of component i not bonded at site A
!	Initialize it to 1 as an initial guess.  We will also use the sum of absolute error as our error
!	measure.  

do i = 1, ncp
	do a = 1, N_sites(i)
		ss(a,i) = 1.0d0 
	end do
end do



do
	if ( (sum_error .le. tol) .or. (attempts .ge. max_eval) ) EXIT
		
	sum_error = 0.0d0
	do i = 1, ncp
		do a = 1, N_sites(i)

			sum = 0.0d0
			do j = 1, ncp
				do b = 1, N_sites(j)

					sum = sum + x(j) * ss(b,j) * Q(i,j,a,b)
				end do
			end do

			ss_new(a,i) = 1.0d0 / (1.0d0 + rhox * sum)
			sum_error = sum_error + abs( ss_new(a,i) - ss(a,i) ) 
			ss_new2(a,i) = amix * ss_new(a,i) + (1.0d0 - amix) * ss(a,i)
			ss(a,i) = ss_new2(a,i)
			
		end do
	end do
	attempts = attempts + 1
end do 

if (attempts .ge. max_eval) then
	write(6,*) 'Xai convergence has not been reached after max evaluation interval'
end if


!	Calculate the temperature derivatives of ss(a,i) 
!	Initialize dss_dbeta to -0.5

do i = 1, ncp
	do a = 1, N_sites(i)
		dss_dbeta(a,i) = -0.5
	end do 
end do

sum_error = 1.0d0
attempts = 1

do
	if ( (sum_error .le. tol) .or. (attempts .ge. max_eval) ) EXIT
	sum_error = 0.0d0
	do i = 1, ncp
		do a = 1, N_sites(i)

			sum = 0.0d0
			do j = 1, ncp
				do b = 1, N_sites(b)

					sum = sum + x(j) * (ss(b,j) * (-TK * dQdT(i,j,a,b)) + dss_dbeta(b,j) * Q(i,j,a,b) )
				end do
			end do

			dss_dbeta_new(a,i) = -ss(a,i) * ss(a,i) * sum * rhox
			sum_error = sum_error + abs( dss_dbeta_new(a,i) - dss_dbeta(a,i) )
			dss_dbeta_new2(a,i) = dss_dbeta_new(a,i) * amix + (1.0d0 - amix) * dss_dbeta(a,i)
			dss_dbeta(a,i) = dss_dbeta_new2(a,i)

		end do
	end do

end do

if (attempts .ge. max_eval) then
	write(6,*) 'dXai/d(Beta) convergence has not been reached after max interval'
end if


!	Calculate the derivative of ss(a,i) wrt rho(m)
!	Initialize dss_drhoi to 0

do i = 1, ncp
	do a = 1, N_sites(i)
		do k = 1, ncp
			dss_drhoi(a, i, k) = 0.0d0
		end do
	end do
end do

sum_error = 1.0d0
attempts = 1

do
	if ( (sum_error .le. tol) .or. (attempts .ge. max_eval) ) EXIT
	sum_error = 0.0d0
	do i = 1, ncp
		do a = 1, N_sites(i)
			do k = 1, ncp

				sum1 = 0.0d0 
				do b = 1, N_sites(k)
					sum1 = ss(b,k) * Q(i,k,a,b) + sum1
				end do

				sum = 0.0d0
				do j = 1, ncp
					do b = 1, N_sites(j)
		
						sum = sum + x(j) * (ss(b,j) * dQdrhoi(i,j,a,b,k) + dss_drhoi(b,j,k) * Q(i,j,a,b) ) 
					end do
				end do

				dss_drhoi_new(a,i,k) = -1.0d0 * ss(a,i) * ss(a,i) * (sum*rhox + sum1) 
			
				sum_error = sum_error + abs( dss_drhoi_new(a,i,k) - dss_drhoi(a,i,k) )
				dss_drhoi_new2(a,i,k) = amix * dss_drhoi_new(a,i,k) + (1.0d0 - amix) * dss_drhoi(a,i,k) 
				dss_drhoi(a,i,k) = dss_drhoi_new2(a,i,k) 
			end do
		end do
	end do
	attempts = attempts + 1
end do

if (attempts .ge. max_eval) then
	write(6,*) 'dXai/d[ rho(i) ] convergence has not been reached after max interval'
end if
			 

!	Calculate the derivative of ss(a,i) wrt rho
!	Initialize dssdrho to 0
do i = 1, ncp
	do a = 1, N_sites(i)
		dssdrho(a,i) = 0.0d0
	end do
end do

sum_error = 1.0d0
attempts = 1

do
	if ( (sum_error .le. tol) .or. (attempts .ge. max_eval) ) EXIT
	sum_error = 0.0d0
	do i = 1, ncp
		do a = 1, N_sites(i)

			sum1 = 0.0d0
			sum = 0.0d0 

			do j = 1, ncp
				do b = 1, N_sites(j)

					sum = sum + x(j) * ( ss(b,j) * dQ_drho(i,j,a,b) + Q(i,j,a,b) * dssdrho(b,j) )

					sum1 = sum1 + x(j) * ss(b,j) * Q(i,j,a,b) 
				end do
			end do

			dssdrho_new(a,i) = -ss(a,i) * ss(a,i) * (sum1 + rhox * sum) 
			sum_error = sum_error + abs( dssdrho(a,i) - dssdrho_new(a,i) )
			dssdrho_new2 = amix * dssdrho_new(a,i) + (1.0d0 - amix) * dssdrho(a,i) 
			dssdrho(a,i) = dssdrho_new2(a,i)
		end do
	end do
	attempts = attempts + 1
end do

if (attempts .ge. max_eval) then
	write(6,*) 'dXai/drho convergence has not been reached after max # of iterations' 
end if

!	Calculating thermodynamic properties due to hydrogen bonding
!	The codes below is almost exactly the same as Dr. Chapman's codes 
!	Initializing the variables



do i = 1, ncp
	prod = 1.0d0
	prod2 = 0.0d0
	prod3 = 0.0d0
		 
	do a = 1, N_sites(i)
		prod = prod * ss(a,i)
		prod2 = dss_dbeta(a,i) / ss(a,i) + prod2 
		prod3 = dssdrho(a,i) / ss(a,i) + prod3 

	end do
	
	ssprod(i) = prod
	dss_dbetaprod(i) = ssprod(i) * prod2
	dssdrhoprod(i) = ssprod(i) * prod3 

end do

do i = 1, ncp
	do k = 1, ncp
		prod4 = 0.0d0 
		do a = 1, N_sites(i)

			prod4 = dss_drhoi(a,i,k) / ss(a,i) + prod4
		
		end do

		dss_drhoiprod(i,k) = ssprod(i)*prod4
	end do
end do



Ahb = 0.0d0
Uhb = 0.0d0
Phb = 0.0d0 


do i = 1, ncp

	sum_deltaA1 = 0.0d0
	sum_deltaU1 = 0.0d0
	sum_deltaP1 = 0.0d0

	do a = 1, N_sites(i)

		sum_deltaA1 = sum_deltaA1 + ss(a,i)
		sum_deltaU1 = sum_deltaU1 + dss_dbeta(a,i)
		sum_deltaP1 = sum_deltaP1 + dssdrho(a,i) * (1.0d0 / ss(a,i) - 0.50d0 )
	end do

	Ahb = Ahb + x(i) * (log(ssprod(i)) - sum_deltaA1 / 2.0d0 + N_sites(i) / 2.0d0)
	Uhb = Uhb + x(i) * (dss_dbetaprod(i) / ssprod(i) - sum_deltaU1 / 2.0d0 )
	Phb = Phb + x(i) * rhox * sum_deltaP1 

	sum_mu1 = 0.0d0
	do j = 1, ncp

		sum_mu = 0.0d0
		do b = 1, N_sites(j)
			sum_mu = sum_mu + dss_drhoi(b,j,i)
		end do

		sum_mu1 = sum_mu1 + x(j) * (dss_drhoiprod(j,i) / ssprod(j) - sum_mu / 2.0d0 )

	end do

	sum_mu2 = 0.0d0 
	do a = 1, N_sites(i)
		sum_mu2 = sum_mu2 + ss(a,i)
	end do
	chptlhb(i) = log(ssprod(i)) + rhox * sum_mu1 - sum_mu2 / 2.0d0 + N_sites(i) / 2.0d0

end do


Hhb = Uhb + Phb
Ghb = Ahb + Phb
Shb = -1.0d0 * Ahb + Uhb






end subroutine