 subroutine test_ghs_derivatives(sitos1, zeta2, zeta3, m, ncp, c, dcdt, dz1dt, dz2dt, dz3dt, g, &
			& rhodgdrho, dgdt, dgdrho) 

!	Subroutine ghs_derivatives
!
!	Program written by P. David Ting using codes written by Walter G. Chapman
!
!	Purpose:
!		Produces derivatives of the hard sphere distribution function used in thermo_props
!		These are: g(i,j) at contact, rho*[dg(i,j)/drho], [dg(i,j)/drho(m)], and [dg(i,j)/dT]

!		
!		The HS distribution function used is the Mansoori, Leland, Carnahan, Starling
!			distribution function for a mixture of hard spheres.  This can be found in 
!			Theory of Simple Liquids by Hansen and McDonald or in Applied Statistical Mechanics
!			by T.M. Reed and K.E. Gubbins, McGraw-Hill, chapter 9.  
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/24/99		P.D. Ting				revised for multicomponent and updated 
!													for fortran 90/95	

implicit none

!	Checking single & double precision 

integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)




!	Declaring calling variables

integer, intent(in) :: ncp									! declaring number of components
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! dhs(i)/sig(1,1) (reduced temp. dependent sphere diameter)
real (KIND=double), intent(in) :: zeta2						! zeta2
real (KIND=double), intent(in) :: zeta3						! zeta3
real (KIND=double), intent(in), dimension(ncp) :: m			! chain length of molecule i
real (KIND=double), intent(in), dimension(ncp, ncp) :: c	! variable lump used in calculations
real (KIND=double), intent(in), dimension(ncp, ncp) :: dcdt !derivative of c wrt T
real (KIND=double), intent(in) :: dz1dt						! derivative of zeta1 wrt T
real (KIND=double), intent(in) :: dz2dt						! derivative of zeta2 wrt T
real (KIND=double), intent(in) :: dz3dt						! derivative of zeta3 wrt T
real (KIND=double), intent(out), dimension(ncp, ncp) :: g	! hard sphere g(r) at contact
real (KIND=double), intent(out), dimension(ncp, ncp) :: rhodgdrho	! rho*[dg(i,j)/drho]
real (KIND=double), intent(out), dimension(ncp, ncp, ncp) :: dgdrho	! [dg(i,j)/drho(m)]
real (KIND=double), intent(out), dimension(ncp, ncp) :: dgdt		! [dg(i,j)/dT]

!real (KIND=double), intent(out), dimension(ncp, ncp) :: gdimer	! hard dimer g(r) at contact
!real (KIND=double), intent(out), dimension(ncp, ncp, ncp) :: dgddrho	! [dghd(i,j)/drho(k)]


!	Declare local parameter
real (KIND=double), parameter :: pi = 3.141592654d0
real (KIND=double) :: pi_over6 = pi / 6.0D0
real (KIND=double) :: z3minus, z3minus_s, z3minus_c, z3minus_f	! z3minus, its squares, cubes, and 4th power 
real (KIND=double), dimension (ncp, ncp) :: cs			! c(i,j) ^2
integer :: i,j, k											! indices



!	Meat starts here

z3minus = 1.0d0 - zeta3
z3minus_s = z3minus * z3minus						! (z3minus)^2
z3minus_c = z3minus_s * z3minus						! (z3minus)^3 
z3minus_f = z3minus_c * z3minus						! (z3minus)^4



forall (i=1:ncp, j=1:ncp)
	cs(i,j) = c(i,j) ** 2
end forall

forall (i=1:ncp, j=1:ncp)

	g(i,j) = 1.0d0 / z3minus + 3.0d0 * c(i,j) * zeta2 / z3minus_s + 2.0d0 * cs(i,j) * zeta2**2 / z3minus_c

	rhodgdrho(i,j) = zeta3 / z3minus_s + 3.0d0 * c(i,j) * (zeta2 / z3minus_s + 2.0d0 * zeta2 * zeta3 / z3minus_c ) &
					& + 2.0d0 * cs(i,j) * (2.0d0 * zeta2** 2 / z3minus_c + 3.0d0 * zeta2** 2 * zeta3 / z3minus_f)

	dgdt(i,j) = dz3dt / z3minus_s + 3.0d0 * c(i,j) * ( dz2dt / z3minus_s + 2.0d0 * zeta2 * dz3dt / z3minus_c) &
					& + 2.0d0 * cs(i,j) * (2.0d0 * zeta2 * dz2dt / z3minus_c + 3.0d0 * zeta2**2 * dz3dt / z3minus_f) &
					& + 3.0d0 * zeta2 * dcdt(i,j) / z3minus_s + 4.0d0 * c(i,j) * zeta2**2 * dcdt(i,j) / z3minus_c

end forall




forall (i=1:ncp, j=1:ncp, k=1:ncp)

	dgdrho(i,j,k) = pi_over6 * m(k) * (sitos1(k)**3 / z3minus_s + 3.0d0 * c(i,j) * (sitos1(k)**2 / z3minus_s &
					& + 2.0d0 * sitos1(k)**3 * zeta2 / z3minus_c ) + 2.0d0 * cs(i,j) * (2.0d0 * sitos1(k)**2 * zeta2 &
					& / z3minus_c + 3.0d0 * sitos1(k)**3 * zeta2**2 / z3minus_f))

end forall


end subroutine



