subroutine test_hs_cs(sitos1, x_spheres, ncp, rho_spheres, dz1dt, dz2dt, dz3dt, TK, Prref, Arref, &
				& Grref, Srref, Urref, Hrref, chptref, pcref)

!	Subroutine hs_cs
!
!	Program written by P. David Ting using codes written by Walter G. Chapman
!
!	Purpose:
!		Calculates the properties of a hard sphere mixture from the Carnahan-Starling EOS
!			All the equations come from Applied Statistical Mechanics by T.M. Reed and K.E. 
!			Gubbins, McGraw-Hill, chapter 9.  
!		This code calculates:
!				residual pressure [p/rho kT]
!				residual internal energy [U/NkT]
!				residual Helmholtz free energy [A/NkT]
!				residual Gibbs free energy [G/NkT]
!				residual entropy [S/Nk]
!				residual enthalpy [H/NkT]
!				residual chemical potential of components [chptl/kT]
!			N is the number of SPHERES
!			
!
!	Record of revisions:
!			Date		Programmer				Description of change
!			====		==========				=====================
!			1997		W.G. Chapman			Thermoprop subroutines
!			7/24/99		P.D. Ting				Revised for multicomponent and updated 
!													for fortran 90/95
!												Included revised dependence (U) on Temperature
!													based on calculations done by Victor Perez	

implicit none

!	Checking single & double precision
integer, parameter :: single = SELECTED_REAL_KIND(p=6, r=37)
integer, parameter :: double = SELECTED_REAL_KIND(p=13)

!	Declaring calling variables
integer, intent(in) :: ncp									! number of components
real (KIND=double), intent(in) :: TK						! Temperature in Kelvin
real (KIND=double), intent(in) :: dz1dt, dz2dt, dz3dt		! derivative of zetas wrt T
real (KIND=double), intent(in), dimension(ncp) :: sitos1	! dhs(i)/sig(1,1) (reduced temp. dependent sphere dia.)
real (KIND=double), intent(in), dimension(ncp) :: x_spheres	! mole fraction of spheres 
real (KIND=double), intent(in) :: rho_spheres				! density of spheres per volume (angstroms)
real (KIND=double), intent(out) :: pcref			! total (in this case config as well) pressure (also Z)
real (KIND=double), intent(out) :: Arref			! residual Helmholtz free energy 
real (KIND=double), intent(out) :: Prref			! residual pressure
real (KIND=double), intent(out) :: Hrref			! residual Enthalpy
real (KIND=double), intent(out) :: Urref			! residual internal energy
real (KIND=double), intent(out) :: Srref			! residual entropy
real (KIND=double), intent(out) :: Grref			! residual Gibbs free energy
real (KIND=double), intent(out), dimension(ncp) :: chptref	! residual chemical potential of component i



!	Declaring local parameter
real (KIND=double), parameter :: pi=3.141592654d0
real (KIND=double) :: sixover_pi = 6.0d0 / pi
real (KIND=double) :: pi_over6 = pi / 6.0d0 


!	Declaring local variables
real (KIND=double) :: zeta0, zeta1, zeta2, zeta3		! Hard sphere zetas 
real (KIND=double) :: z3, z3s, z3c						! 1 - zeta3, (1-zeta3)^2, and (1-zeta3)^3
integer :: i											! Index counters
real (KIND=double) :: zeta2s, zeta2c					! (zeta2)^2 and (zeta2)^3
real (KIND=double) :: zeta3s, zeta3c					! (zeta3)^2 and (Zeta3)^3
real (KIND=double) :: dArrefdT							! d(Ar,ref)/d(T)
real (KIND=double) :: Gcref								! configurational Gibbs free energy
real (KIND=double) :: Scref								! configurational entropy
real (KIND=double) :: Acref								! configurational Helmholtz free energy
real (KIND=double) :: Hcref								! configurational enthalpy
real (KIND=double) :: Ucref								! configurational internal energy
real (KIND=double), dimension(ncp) :: sitos1s, sitos1c	! square and cube of sitos1

!	Here's the meat

!	Calculating zeta terms for hard spheres.  This is the same as zetas calculated in zeta_s
!		except that the zeta in zeta_s are calculated from molecules (chain) perspective while
!		the below calculations are from the segment (spheres) perspective.

zeta0 = 1.0d0
zeta1 = 0.0d0
zeta2 = 0.0d0
zeta3 = 0.0d0

forall (i=1:ncp)
	sitos1s(i) = sitos1(i) * sitos1(i)
	sitos1c(i) = sitos1s(i) * sitos1(i)
end forall

do i = 1, ncp
	zeta1 = x_spheres(i) * sitos1(i) + zeta1
	zeta2 = x_spheres(i) * sitos1s(i) + zeta2
	zeta3 = x_spheres(i) * sitos1c(i) + zeta3
end do

zeta0 = pi_over6 * rho_spheres * zeta0
zeta1 = pi_over6 * rho_spheres * zeta1
zeta2 = pi_over6 * rho_spheres * zeta2
zeta3 = pi_over6 * rho_spheres * zeta3
	  
z3 = 1.0d0 - zeta3
!write(6,*)'z3 = ',z3
z3s = z3 * z3
z3c = z3s * z3

zeta2s = zeta2 * zeta2
zeta2c = zeta2s * zeta2

zeta3s = zeta3 * zeta3
zeta3c = zeta3s * zeta3


!	Refer to the Hard Sphere terms derivations paper
!	Pressure calculations 

pcref = sixover_pi * (zeta0 / z3 + 3.0d0 * zeta1 * zeta2 / z3s + 3.0d0 * zeta2c / z3c &
		& - zeta3 * zeta2c / z3c) / rho_spheres

Prref = pcref - 1.0d0

!	Helmholtz free energy calculations

Arref = sixover_pi * (( zeta2c + 3*zeta1*zeta2*zeta3 - 3*zeta1*zeta2*zeta3s)/zeta3/z3s &
		& - (zeta0 - zeta2c / zeta3s) * log(z3)) / rho_spheres

!	Gibbs free energy calculations

Grref = Arref + Prref


!	Entropy calculation based on Srref = -d(Arref)/dT holding volume and n constant
!	[d(Arref)/dT]/nk is called dArrefdT

dArrefdT = Arref + sixover_pi * TK / rho_spheres * (  (zeta0 - zeta2c / zeta3s) * dz3dt / z3 &
			& +  (3.0d0 * zeta2s / zeta3s * dz2dt - 2.0d0 * zeta2c / zeta3c * dz3dt) * log(z3) &
			& - ( zeta2c + 3.0D0*zeta1*zeta2*zeta3 - 3.0D0*zeta1*zeta2*zeta3s )/zeta3s/z3c &
			& * (z3*dz3dt - 2.0D0 * zeta3*dz3dt) &
			& + 3.0d0 * (zeta2s*dz2dt + (zeta2*zeta3*dz1dt + zeta1*zeta3*dz2dt + zeta1*zeta2*dz3dt) &
			& - (zeta2*zeta3s*dz1dt + zeta1*zeta3s*dz2dt + 2.0d0*zeta1*zeta2*zeta3*dz3dt))/zeta3/z3s  )

Srref = -dArrefdT

!	Internal energy from general thermo relationships
Urref = Arref + Srref

!	Enthalpy from general thermo relationships
Hrref = Prref + Urref 

!	Chemical potential calculations

forall (i=1:ncp)
	chptref(i) = pi/6.0d0 * pcref * rho_spheres * sitos1c(i) - log(z3) & 
				& + 3.0d0 * (zeta2 * sitos1(i) + zeta1 * sitos1s(i)) /z3 &
				& + 9.0d0/2.0d0 * zeta2s*sitos1s(i) / z3s &
				& + 3.0d0 * zeta2s * sitos1s(i) / zeta3s * (log(z3) + zeta3/z3 - zeta3s/2.0d0/z3s) &
				& - zeta2c * sitos1c(i) / zeta3c * (2.0d0 * log(z3) + zeta3 * (2.0d0-zeta3) / z3)
end forall 

end subroutine
